/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains Date Field Classes.
 * @author Erkan Cengiz
 * @version $Id: Date.js 184 2008-07-30 14:35:18Z mustafa $
 */


var __currCalendarBox = null;

/**
 * @class
 * Represents a DateText. It wraps any HTML DOM element that can contains a text node as
 * a child node. The purpose of this class is to create a text node under the wrapped DOM element
 * and format its value as a date.
 *
 * @extends TextNode
 * @param node the DOM Element node that is wrapped by this object
 */
function DateText(node) {

  /**
   * DateBox Properties Object.
   *
   *<br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b><a name="value">value</a></b></td><td>The value of the DateText</td></tr>
   * <tr><td valign="top"><b>valueFormat</b></td><td>The format of the value. See <a href="DateManager.html#dateFormatingSymbols">Date Formatting Symbols</a>
   * <tr><td><b><a name="displayFormat">displayFormat</a></b></td><td> The representation format of the date value. See <a href="DateManager.html#dateFormatingSymbols">Date Formatting Symbols</a> </td></tr>
   * <tr><td><b>defaultDate</b></td><td> The default date value to be used when <b>value</b> property is empty </td></tr>
   * </table>
   *
   */
  this.props = null;
  /** @private */

  try {
    this.base = TextBox;
    this.base(node);
    this.needsValueCloning = true;
  } catch (e) {
    /** @private */
    this.base = TextNode;
    this.base(node);
  }

  /** @type boolean */
  this.isDateText = true;

  /**
   * Initializes this DateText
   */
  this.init = function() {
    this.setValue(this.props.value || this.props.defaultDate);
  };

  this.setValue = function(value) {
    if (!value) return;
    var d = DateMgr.parseDate(value,this.props.valueFormat);
    var v = DateMgr.formatDate(d,this.props.displayFormat);
    this._setValue(v);
  };

  /** @private */
  this.ajaxRefreshHandler = function(rec,nodeName,attr) {

    if (!nodeName) nodeName = this.name;

    if (!attr) attr = "value";
    if (attr == "value") this.setValue("");
    else if (attr == "href") this.setHref("");

    if (rec) {
      if (!rec.owner.hasField(nodeName)) {
        throw new Exception("RESPONSE_FIELD_MISSING",nodeName);
      }

      if (attr == "value") {
       this.setValue( rec.get(nodeName) );

      } else if (attr == "href") this.setHref( rec.get(nodeName) );
    }

  };



};

/**
 * @class
 * Represents a DateBox. It wraps a <code>TEXT INPUT</code> element, which is used for typing in the date,
 * and a <code>HIDDEN INPUT</code> element to keep the value formatted convenient to send to the server.
 *
 * @extends ProcessField
 * @param node the DOM Element node that is wrapped by this object
 */

function DateBox(node) {

  /**
   * DateBox Properties Object.
   *
   *<br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b>valueFormat</b></td><td>The format of the value. See <a href="DateManager.html#dateFormatingSymbols">Date Formatting Symbols</a>
   * </td></tr>
   * <tr><td><b><a name="displayFormat">displayFormat</a></b></td><td> The representation format of the date value. See <a href="DateManager.html#dateFormatingSymbols">Date Formatting Symbols</a> </td></tr>
   * <tr><td><b><a name="submitFormat">submitFormat</a></b></td><td> The format of the submit value.See <a href="DateManager.html#dateFormatingSymbols">Date Formatting Symbols</a> </td></tr>
   * <tr><td><b>monthFormat</b></td><td> The Format of the Month. The formating options are : "Long Text", "Short Text", "Number" </td></tr>
   * <tr><td><b>excludeWeekends</b></td><td> The Flag to make weekend days invisible on the dialog </td></tr>
   * <tr><td><b>defaultDate</b></td><td> The default date value to be used when {@link #intialValue} is empty </td></tr>
   * <tr><td><b>fromDate</b></td><td> The Minimum date that can be selected on the dialog </td></tr>
   * <tr><td><b>toDate</b></td><td> The maximum date that can be selected on the dialog </td></tr>
   * <tr><td><b>nonBlank</b></td><td>  See <a href="ProcessField.html#nonBlank">ProcessField.props.nonBlank</a> </td></tr>
   * <tr><td><b>validValues</b></td><td> See <a href="ProcessField.html#validValues">ProcessField.props.validValues</a></td></tr>
   * <tr><td><b>valHandler</b></td><td> See <a href="ProcessField.html#valHandler">ProcessField.valHandler()</a></td></tr>
   * <tr><td><b>valErrorHandler</b></td><td> See <a href="ProcessField.html#valErrorHandler">ProcessField.props.valErrorHandler</a></td></tr>
   * </table>
   *
   */
  this.props = null;

  try {
    this.base = ProcessField;
    this.base(node);
  } catch (e) {
    this.base = MoreMotionObject;
    this.base(node);
  }
  /** @type boolean */
  this.isDateBox = true;

  /** @private */
  this.wrapElements = function() {
    this.elm = this.node;
    var parentNode = this.node.tagName == "INPUT" ? this.node.parentNode : this.node;
    var elms = parentNode.getElementsByTagName("input");
    if (elms && elms.length) {
      for (var i = 0; i < elms.length; i++) {
        var elm = elms[i];
        if (elm.name.startsWith("pf_") || elm.name == this.name) {
          this.selm = elm;
        } else if (elm.name == "__t") {
          this.elm = elm;
        }
      }
    }
  };

  this.wrapElements();

  /**
   * Initializes this DateBox
   */
  this.init = function() {
    var value = this.getIValue();
    if (!value && this.props.defaultDate) {
      var d = DateMgr.parseDate(this.props.defaultDate,this.props.valueFormat);
      value = DateMgr.formatDate(d,this.props.submitFormat);
      this.setIValue(value);
    }
    this.setValue(value, this.props.valueFormat);
  };

  /** @private */
  this.changed = function() {

    if (this.elm.value == "") {
      this.selm.value = "";
      return;
    }

    var cDate = DateMgr.parseDate(this.elm.value,this.props.displayFormat);
    if (cDate == "NaN") {
      alert( OMgr.resource("INVALID_DATE",this.props.displayFormat) );
      this.elm.focus();
      this.elm.select();
      return false;
    }

    var fDate = DateMgr.parseDate(this.props.fromDate,"yyyy.mm.dd");
    var tDate = DateMgr.parseDate(this.props.toDate,"yyyy.mm.dd");

    if (cDate < fDate || cDate > tDate) {
      alert( OMgr.resource("DATE_OUT_OF_RANGE", DateMgr.formatDate(fDate,this.props.displayFormat), DateMgr.formatDate(tDate,this.props.displayFormat)) );
      this.elm.focus();
      this.elm.select();
      return false;
    }
    this.selm.value = DateMgr.formatDate(cDate,this.props.submitFormat);
    this.elm.value = DateMgr.formatDate(cDate,this.props.displayFormat);

    this.setModified();
  };


  /**
   * Empties the all wrapped input fields and sets the value of the DateBox to null.
   */
  this.clear = function() {
    this.elm.value = "";
    this.selm.value = "";
  };


  /**
   * Sets the value of this DateBox. This method ensures that
   * the value of the <code>TEXT INPUT</code> element is formatted
   * according to the {@link #displayFormat} property and the value of the <code>HIDDEN INPUT</code> element
   * is formatted according to the {@link #submitFormat} property.
   * @param value The new value. The format of the value should conform to the format defined with
   * @param valueFormat The format of the new value. If not specified props.submitFormat is used.
   * {@link #dateFormatingSymbols} property.
   */
  this.setValue = function(value, valueFormat) {
    if (value == "" && this.props.defaultDate == "") {
      this.elm.value = "";
      this.selm.value = "";
      return;
    }

    if (value == "") value = this.props.defaultDate;
    if (!value) return;
    var d = DateMgr.parseDate(value, valueFormat || this.props.submitFormat);
    this.elm.value = DateMgr.formatDate(d,this.props.displayFormat);
    this.selm.value = DateMgr.formatDate(d,this.props.submitFormat);
  };

  this.getValue = function() {
    return this.selm.value;
  };

  /** @private */
  this.validate = function() {
    if (this.boolProp("nonBlank") && this.checkValueIsAvailable(this.selm,this.elm) == false) return false;
    if (this.props.validValues && this.checkHasValidValue(this.selm,this.elm) == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
  };

  /** @private */
  this._validate = function(handler) {

    if (this.elm.value == "")  return true;

    var cDate = DateMgr.parseDate(this.elm.value,this.props.displayFormat);
    if (cDate == "NaN") {
      return this.notify(this.elm, "INVALID_DATE", OMgr.resource("INVALID_DATE"));
    }

    var fDate = DateMgr.parseDate(this.props.fromDate,"yyyy.mm.dd");
    var tDate = DateMgr.parseDate(this.props.toDate,"yyyy.mm.dd");

    if (cDate < fDate || cDate > tDate) {
      var msg = OMgr.resource("DATE_OUT_OF_RANGE",DateMgr.formatDate(fDate,this.props.displayFormat), DateMgr.formatDate(tDate,this.props.displayFormat));
      return this.notify(this.elm,"DATE_OUT_OF_RANGE",msg);
    }

  }

};

/**
 * @class
 * Represents a DateDialog Process Field which wraps three <code>SELECT</code> elements,
 * a readonly <code>TEXT INPUT</code> element
 * and a <code>HIDDEN INPUT</code> element to provide a controlled input field for entering date data.
 * The three <code>SELECT</code> elements are for selecting Day, Month and Year portions of the date seperately.
 * <p>The <code>HIDDEN INPUT</code> element contains the date to sent to the server and therefore this class
 * reflects all the changes made to the <code>SELECT</code> elements to immediatelly this field.
 * @extends ProcessField
 * @param node the DOM Element node that is wrapped by this object
 */

function DateDialog(node) {


  /**
   * DateDialog Properties Object.
   *
   *<br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b>valueFormat</b></td><td>The format of the value. <p> <b>The formatting symbols: </b><p>
   *    <ul>
   *      <li type="square"><code>yy&nbsp;&nbsp;</code> - Two-digit year</li>
   *      <li type="square"><code>yyyy</code> - Four-digit year</li>
   *      <li type="square"><code>mm&nbsp;&nbsp;</code> - Month</li>
   *      <li type="square"><code>dd&nbsp;&nbsp;</code> - Day</li>
   *      <li type="square"><code>ddd&nbsp;</code> - Sort form Day</li>
   *      <li type="square"><code>dddd</code> - Long form Day</li>
   *      <li type="square"><code>hh&nbsp;&nbsp;</code> - Hour</li>
   *      <li type="square"><code>nn&nbsp;&nbsp;</code> - Minute</li>
   *      <li type="square"><code>ss&nbsp;&nbsp;</code> - Second</li>
   *      <li type="square"><code>Integer</code> - Integer</li>
   *      <li type="square"><code>Parse</code> - The value is parsed by auto detecting the value format</li>
   *    </ul></td></tr>
   * <tr><td><b>submitFormat</b></td><td> The format of the submit value. See {@link #dateFormatingSymbols} </td></tr>
   * <tr><td><b>monthFormat</b></td><td> The Format of the Month. The formating options are : "Long Text", "Short Text", "Number" </td></tr>
   * <tr><td><b>excludeWeekends</b></td><td> The Flag to make weekend days invisible on the dialog </td></tr>
   * <tr><td><b>defaultDate</b></td><td> The default date value to be used when {@link #intialValue} is empty </td></tr>
   * <tr><td><b>fromDate</b></td><td> The Minimum date that can be selected on the dialog </td></tr>
   * <tr><td><b>toDate</b></td><td> The maximum date that can be selected on the dialog </td></tr>
   *
   * <tr><td><b>nonBlank</b></td><td>  See <a href="ProcessField.html#nonBlank">ProcessField.props.nonBlank</a> </td></tr>
   * <tr><td><b>validValues</b></td><td> See <a href="ProcessField.html#validValues">ProcessField.props.validValues</a></td></tr>
   * <tr><td><b>valHandler</b></td><td> See <a href="ProcessField.html#valHandler">ProcessField.valHandler()</a></td></tr>
   * <tr><td><b>valErrorHandler</b></td><td> See <a href="ProcessField.html#valErrorHandler">ProcessField.props.valErrorHandler</a></td></tr>
   *
   * <tr><td valign="top"><b>onChange</b></td><td> The name of the event handler to be called when date is changed by the user. When called the handler
   * will receive this DateDialog object as the parameter. </td></tr>
   * <tr><td><b>onDayChange</b></td><td> The name of the event handler to be called when day is changed </td></tr>
   * <tr><td><b>onMonthChange</b></td><td> The name of the event handler to be called when month is changed </td></tr>
   * <tr><td><b>onYearChange</b></td><td> The name of the event handler to be called when year is changed </td></tr>
   * </table>
   *
   */
  this.props = null;

  try {
    this.base = ProcessField;
    this.base(node);
    this.needsValueCloning = true;
  } catch (e) {
    this.base = MoreMotionObject;
    this.base(node);
  }
  /** @type boolean */
  this.isDateDialog = true;


  /**
   * The HTMLElement object of <code>HIDDEN TEXT</code> element that keeps the
   * value to submit to the server.
   * @type HTMLElement
   */
  this.selm = null; //-<

  /** @private */
  this.welm = null; //-<

  /**
   * The HTMLElement object of <code>SELECT</code> element that keeps the Day value
   * @type HTMLElement
   */
  this.delm = null; //-<

  /**
   * The HTMLElement object of <code>SELECT</code> element that keeps the Month value
   * @type HTMLElement
   */
  this.melm = null; //-<

  /**
   * The HTMLElement object of <code>SELECT</code> element that keeps the Year value
   * @type HTMLElement
   */
  this.yelm = null; //-<

  /**
   * The HTMLElement object of <code>SELECT</code> element that keeps the Day of Week value
   * @type HTMLElement
   */
  this.telm = null; //-<


  /** @private */
  this.wrapElements = function() {

    var elms = this.node.getElementsByTagName("select");
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      if (elm.name == "__Year") this.yelm = elm;
      else if (elm.name == "__Month") this.melm = elm;
      else if (elm.name == "__Day") this.delm = elm;
    }

    elms = this.node.getElementsByTagName("input");
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      if (elm.name == "__DayText") this.telm = elm;
      else if (elm.name == "__Work") this.welm = elm;
      else if (elm.name.startsWith("pf_") || elm.name == this.name) this.selm = elm;
    }

  };

  /** @private */
  this.wrapElements();

  /**
   * Initializes this DateDialog
   */
  this.init = function() {

    var value = this.getIValue();
    if (!value && this.props.defaultDate) {
      var d = DateMgr.parseDate(this.props.defaultDate,this.props.valueFormat);
      value = DateMgr.formatDate(d,this.props.submitFormat);
      this.setIValue(value);
    }
    this.reformat(value);

    // this.reformat(this.getIValue());
  };

  /** @private */
  this.reformat = function(value) {

    var empty = !this._setValue(value || this.getIValue());

    this.fillYearOptions();
    this.fillMonthOptions();
    this.fillDayOptions();

    if (empty) this.setEmpty();

  };


  /** @private */
  this.fillAll = function() {
    try {
      if (this.delm.value == "") this.delm.value = this.delm.options[1].value;
      if (this.melm.value == "") this.melm.value = this.melm.options[1].value;
      if (this.yelm.value == "") this.yelm.value = this.yelm.options[1].value;
    } catch (e) {}
  };

  /** @private */
  this.emptyAll = function() {
    this.delm.value = "";
    this.melm.value = "";
    this.yelm.value = "";
    if (this.telm) this.telm.value = "";
    this.selm.value = "";
    this.welm.value = "";
  };


  /** @private */
  this.updateDates = function(t,value) {

    if (value == "") {
      this.emptyAll();
      return false;
    } else {
      this.fillAll();
    }

    var theDate = new Date();

    if (t == 0)
      theDate.setTime(value);
    else {
      theDate.setTime(this.welm.value);

      var y = theDate.getFullYear();
      var m = theDate.getMonth();
      var d = theDate.getDate();

      if (t == 1) y = value;
      else if (t == 2) m = value;
      else d = value;

      if (d > DateMgr.nbrOfDays(y, m))
        d = DateMgr.nbrOfDays(y, m);

      theDate = new Date(y,m,d);
    }

    this.welm.value = theDate.getTime();

    this.selm.value = DateMgr.formatDate(theDate,this.props.submitFormat);

    if (this.telm != null)
      this.telm.value = DateResource.days[theDate.getDay()];

    this.isEmpty = false;

    return true;
  };

  /**
   * Sets the year value of the dialog
   * @param value The Year value as integer
   */
  this.setYear = function(value) {
    this.updateDates(1,value);
  };

  /**
   * Sets the month value of the dialog
   * @param value The Month value as integer (0 is the January, 11 is the December).
   */
  this.setMonth = function(value) {
    this.updateDates(2,value);
  };

  /**
   * Sets the day value of the dialog
   * @param value The Day value as integer
   */
  this.setDay = function(value) {
    this.updateDates(3,value);
  };

  /**
   * Sets the date value of the dialog
   * @param theDate The Day value as integer
   */
  this.setDate = function(theDate) {
    if (theDate.getTime) this.updateDates(0,theDate.getTime);
    else this.updateDates(0,theDate);
  };

  /**
   * Returns the current date as Date object
   * @type Date
   */
  this.getDate = function() {
    return this.getCurrentDate();
  };

  /**
   * Returns the day of week as string
   * @type String
   */
  this.getWeekDayAsString = function() {
    return DateResource.days[this.getCurrentDate().getDay()];
  };

  /**
   * Returns the day as string
   * @type String
   */
  this.getDayAsString = function() {
    return DateResource.days[this.getCurrentDate().getDay()];
  };

  /**
   * Returns the month as string
   * @type String
   */
  this.getMonthAsString = function() {
    return DateResource.months[this.getCurrentDate().getMonth()];
  };

  /** @private */
  this.setEmpty = function() {
    this.selm.value = "";
    this.welm.value = "";
    this.delm.value = "";
    this.melm.value = "";
    this.yelm.value = "";
    if (this.telm) this.telm.value = "";
    this.isEmpty = true;
  };

  /** @private */
  this.getCurrentDate = function() {
    var theDate = new Date();
    theDate.setTime(this.welm.value);
    return theDate;
  }

  /** @private */
  this.fillDayOptions = function() {

    for (var i = this.delm.options.length; i >= 0; i--)
      this.delm.options[i] = null;

    var opt = new Option( "" );
    opt.value = "";
    this.delm.options[0] = opt;

    var cDate = this.getCurrentDate();
    var cYear = cDate.getFullYear();
    var cMonth = cDate.getMonth();
    var cDay = cDate.getDate();

    var fDate = DateMgr.parseDate(this.props.fromDate,"yyyy.mm.dd");
    var tDate = DateMgr.parseDate(this.props.toDate,"yyyy.mm.dd");

    var fYear = fDate.getFullYear();
    var tYear = tDate.getFullYear();

    var fd = ( (cYear == fYear) && (cMonth == fDate.getMonth()) ) ? fDate.getDate() : 1;
    var td = ( (cYear == tYear) && (cMonth == tDate.getMonth()) ) ? tDate.getDate() : DateMgr.nbrOfDays(cYear, cMonth);

    var firstDay = -1;
    var dayFound = false;
    var j = 0;
    for (var i = fd; i <= td; i++) {
      if (!this.props.excludeWeekends || !DateMgr.isWeekendDay(cYear, cMonth, i)) {
        if (firstDay == -1) firstDay = i;
        var opt = new Option( i );
        opt.value = i;
        this.delm.options[++j] = opt;
        if (cDay == i) dayFound = true;
      }
    }

    if (!this.isEmpty) {
      if (dayFound)
        this.delm.value = cDay;
      else
        this.updateDates(3,firstDay);
    }
  }

  /** @private */
  this.fillMonthOptions = function() {

    for (var i = this.melm.options.length; i >= 0; i--)
      this.melm.options[i] = null;

    var opt = new Option( "" );
    opt.value = "";
    this.melm.options[0] = opt;

    var cDate = this.getCurrentDate();
    var cYear = cDate.getFullYear();
    var cMonth = cDate.getMonth();

    var fDate = DateMgr.parseDate(this.props.fromDate,"yyyy.mm.dd");
    var tDate = DateMgr.parseDate(this.props.toDate,"yyyy.mm.dd");

    var fYear = fDate.getFullYear();
    var tYear = tDate.getFullYear();

    var fm = (cYear == fYear) ? fDate.getMonth() : 0;
    var tm = (cYear == tYear) ? tDate.getMonth() : 11;

    if (!this.isEmpty) {
      if ( !( (fm <= cMonth) && (cMonth <= tm) ) ) {
        cMonth = tm;
        this.updateDates(2,cMonth);
      }
    }

    var j = 0;
    for (var i = fm; i <= tm; i++) {
      if (this.props.monthFormat == "Long Text")
        var opt = new Option( DateResource.months[i] );

      else if (this.props.monthFormat == "Short Text")
        opt = new Option( DateResource.shortMonths[i] );

      else
        opt = new Option( i+1 );

      opt.value = i;
      j++;
      this.melm.options[j] = opt;
    }
    this.melm.value = cMonth;
  }

  /** @private */
  this.fillYearOptions = function(id,sbmformat,fromdate,todate) {

    for (var i = this.yelm.options.length; i >= 0; i--)
      this.yelm.options[i] = null;

    var opt = new Option( "" );
    opt.value = "";
    this.yelm.options[0] = opt;

    var cYear = this.getCurrentDate().getFullYear();

    var fDate = DateMgr.parseDate(this.props.fromDate,"yyyy.mm.dd");
    var tDate = DateMgr.parseDate(this.props.toDate,"yyyy.mm.dd");

    var fYear = fDate.getFullYear();
    var tYear = tDate.getFullYear();

    if (!this.isEmpty) {
      if ( !( (fYear <= cYear) && (cYear <= tYear) ) ) {
        cYear = fYear  ;
        this.updateDates(1,cYear);
      }
    }

    var j = 0;
    for (var i = fYear; i <= tYear; i++) {
      var opt = new Option(i);
      opt.value = i;
      j++;
      this.yelm.options[j] = opt;
    }
    this.yelm.value = cYear;
  }

  /** @private */
  this.dayChanged = function(hasFocus) {
    this.updateDates(3,this.delm.value);
    if (this.props.onDayChange) this.runHandler("onDayChange");
    if (this.props.onChange) this.runHandler("onChange");
    if (hasFocus) this.setModified();
  }

  /** @private */
  this.monthChanged = function(hasFocus) {
    if (this.updateDates(2,this.melm.value)) {
      this.fillDayOptions();
    }
    if (this.props.onMonthChange) this.runHandler("onMonthChange");
    if (this.props.onChange) this.runHandler("onChange");
    if (hasFocus) this.setModified();
  }

  /** @private */
  this.yearChanged = function(hasFocus) {
    if (this.updateDates(1,this.yelm.value)) {
      this.fillMonthOptions();
      this.fillDayOptions();
    }
    if (this.props.onYearChange) this.runHandler("onYearChange");
    if (this.props.onChange) this.runHandler("onChange");
    if (hasFocus) this.setModified();
  }


  /**
   * Empties all the wrapped input fields and sets the value of the dialog to null.
   */
  this.clear = function() {
    this.setEmpty();
  };

  /**
   * Sets the value of this DateDialog. After the value is set, the Day, Month and Year <code>SELECT</code> elements
   * will be rearranged to display the new value to the user.
   * @param value The new value. The format of the value should conform to the format defined with
   * {@link #dateFormatingSymbols} property.
   */
  this.setValue = function(value) {
    this.reformat(value);
  };

  this.getValue = function() {
    return this.selm.value;
  };

  /** @private */
  this._setValue = function(value) {

    var theDate;
    if (value == "") {
      if (!this.props.defaultDate) {
        this.setEmpty();
        return false;
      }
      theDate = DateMgr.parseDate(this.props.defaultDate,this.props.valueFormat);
    } else {
      theDate = DateMgr.parseDate(value,this.props.valueFormat);
      if (theDate == "NaN") {
        alert( OMgr.resource("INVALID_DATE",value) );
        return false;
      }
    }

    this.updateDates(0,theDate.getTime());

    return true;
  };

  /** @private */
  this.validate = function() {
    if (this.boolProp("nonBlank") && this.checkValueIsAvailable(this.welm,this.delm) == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
  };


};

/**
 * @class
 * Represents a CalendarBox. It wraps a {@link DateBox} and a opener button.
 * This class uses "The DHTML Javascript Calendar"
 * from <a target="_blank" href="http://www.dynarch.com/projects/calendar/">dynaarch</a>.
 * </p>&nbsp;
 *
 * @extends ProcessField
 * @param node the DOM Element node that is wrapped by this object
 */

function CalendarBox(node) {

  /**
   * CalendarBox Properties Object.
   *
   *<br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b>showAt</b></td><td>The position of the calendar div in <code>x:y</code> format.
   * Additionally value "1" can be used to open the div under the DateBox and value "2" to open the opener button</td></tr>
   * <tr><td><b>firstDayOfWeek</b></td><td>A number between 0 ("Sunday") to 6 ("Saturday") as the first day of the week</td></tr>
   * <tr><td><b>showsTime</b></td><td>Show the time Flag</td></tr>
   * <tr><td><b>time24</b></td><td>24 hours Flag</td></tr>
   * <tr><td><b>weekNumbers</b></td><td>Week Numbers Flag</td></tr>
   * <tr><td><b>showOtherMonths</b></td><td>Show Other Months Flag</td></tr>
   * <tr><td><b>singleClick</b></td><td>Single Click Flag</td></tr>
   * <tr><td><b>statusBarDateFormat</b></td><td>
   * See <a target="_blank" href="http://www.dynarch.com/demos/jscalendar/doc/html/reference.html#node_sec_5.3.5">DHTML Calendar Widget Help</a> </td></tr>
   * <tr><td><b>onSelectHandler</b></td><td>The Javascript function to call when a date is selected on the calendar.
   * The function receives the <a target="_blank" href="http://www.dynarch.com/demos/jscalendar/doc/html/reference.html#node_sec_4">Calendar Object</a>
   * associated with this object as the parameter. If the function returns <code>false</code> then the date selection is refused and
   * the calendar is not closed.</td></tr>
    * <tr><td><b>dateStatusHandler</b></td><td>The Javascript function to call to determine the status of the given date
    * during building a calendar view. The function receives a JS Date object as the parameter. The function may return
    * <code>true</code>, <code>false</code> or a CSS class name</td></tr>
   * </table>
   *
   */
  this.props = null;

  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isCalendarBox = true;

  this.datebox = null;

  this.CAL_FORMAT = "%Y.%m.%d %H:%M";

  /** @private */
  this.wrapElements = function() {
    var dboxNode = OMgr.findChildNode(this.node,null,"mo:type","DateBox");
    if (dboxNode) {
      this.datebox = OMgr.getObject(dboxNode);
    }

    if (this.props.dateStatusHandler) {
      this.userDateStatusHandler = new EventHandler(this.props.dateStatusHandler,"dateStatusHandler");
    }

    if (this.props.onSelectHandler) {
      this.userOnSelectHandler = new EventHandler(this.props.onSelectHandler,"onSelectHandler");
    }

  };

  this.wrapElements();

  this.getValue = function() {
    return this.datebox.getValue();
  }

  this.setValue = function(value) {
    this.datebox.setValue(value);
  }

  this.dateSelectHandler = function(cal, date) {

   if(this.userOnSelectHandler && cal.dateClicked && cal.singleClick) {
     cal.selectedDate = date;
     if (this.userOnSelectHandler.run(cal) == false) {
       return false;
     }
   }

   this.datebox.elm.value = DateMgr.convertDate(date,"yyyy.mm.dd hh:nn",this.datebox.props.displayFormat);
   this.datebox.selm.value = DateMgr.convertDate(date,"yyyy.mm.dd hh:nn",this.datebox.props.submitFormat);

   this.datebox.setModified();

   if (!(cal.dateClicked && cal.singleClick)) {
     return false;
   }

   cal.callCloseHandler();

  };

  this.dateStatusHandler = function(date) {

   if(this.userDateStatusHandler) {
     var userResult = this.userDateStatusHandler.run(date);
     if(userResult) {
       return userResult;
     }
   }
   var minDate = date.getTime() < DateMgr.parseDate(this.datebox.props.fromDate,"yyyy.mm.dd").getTime();
   var maxDate = date.getTime() > DateMgr.parseDate(this.datebox.props.toDate,"yyyy.mm.dd").getTime();
   var valid = maxDate == true || minDate == true;

   return valid;
  };

  this.show = function() {

    if (_dynarch_popupCalendar != null) {
      _dynarch_popupCalendar.hide();
    }

    var initialDate = null;
    if (this.datebox.elm.value) {
      initialDate = DateMgr.parseDate(this.datebox.elm.value,this.datebox.props.displayFormat);
      if (initialDate == "NaN") initialDate = null;
    }

    if (!initialDate) {
      if(this.datebox.props.defaultDate) {
        initialDate = DateMgr.parseDate(this.datebox.props.defaultDate,"yyyy.mm.dd");
      }
      else {
        initialDate = DateMgr.parseDate(this.datebox.props.fromDate,"yyyy.mm.dd");
      }
    }

    if(initialDate < DateMgr.parseDate(this.datebox.props.fromDate,"yyyy.mm.dd")) {
      initialDate = DateMgr.parseDate(this.datebox.props.fromDate,"yyyy.mm.dd");
    }

    var cal = new Calendar(1, initialDate, DateMgr.calendarSelectHandler, DateMgr.calendarCloseHandler);
    cal.setDateStatusHandler(DateMgr.calendarDateStatusHandler);

    cal.firstDayOfWeek = this.charProp("firstDayOfWeek","1") * 1;
    cal.showsTime = this.boolProp("showsTime") || this.boolProp("showTime") ;
    cal.weekNumbers = this.boolProp("weekNumbers");
    cal.time24 = this.boolProp("time24");
    cal.showsOtherMonths = this.boolProp("showsOtherMonths");
    cal.singleClick = this.boolProp("singleClick");
    if(this.props.statusBarDateFormat) {
      cal.setTtDateFormat(this.props.statusBarDateFormat);
    }

    _dynarch_popupCalendar = cal;
    cal.setRange(DateMgr.parseDate(this.datebox.props.fromDate,"yyyy.mm.dd").getFullYear(),DateMgr.parseDate(this.datebox.props.toDate,"yyyy.mm.dd").getFullYear());
    cal.create();

   _dynarch_popupCalendar.setDateFormat(this.CAL_FORMAT);

   var showAt = this.props.showAt;
   if(showAt == "1" || showAt.startsWith("1.")) {
     var node = OMgr.getParentNode(this.datebox.node,"tr");
     if (node) node = OMgr.findChildNode(node,null,"mo:type","DateBox");
     _dynarch_popupCalendar.showAtElement(node, "Br");

   } else if(showAt == "2" || showAt.startsWith("2.")) {
     var node = OMgr.getParentNode(this.datebox.node,"tr");
     if (node) node = OMgr.findChildNode(node,null,"mo:type","CalendarOpener");
     _dynarch_popupCalendar.showAtElement(node, "Br");

   } else {
     var calX = showAt.before(":",null);
     var calY = showAt.after(":",null);
     if (calX && calY) _dynarch_popupCalendar.showAt(calX,calY);
   }

   _dynarch_popupCalendar.element.style.zIndex = "1000";

    return false;
  };

}

/**
 * @class
 * Manager class that provides static methods to manage the {@link DateBox} and {@link DateDialog} classes. A static object of this class
 * is avaialable with name <CODE>DateMgr</CODE>.
 * <PRE>var DateMgr = new DateManager();</PRE>
 *
 * <p>
 *
 * <b><a name="dateFormat">The Date Formating Symbols</a></b><p>
 * The Date Formatting Symbols used by the {@link #parseDate} and {@link #formatDate} methods of this are as follows:
 *    <ul>
 *      <li><code>yy&nbsp;&nbsp;</code> - Two-digit year</li>
 *      <li><code>yyyy</code> - Four-digit year</li>
 *      <li><code>mm&nbsp;&nbsp;</code> - Two-digit month</li>
 *      <li><code>mmm&nbsp;</code> - Month name (short form)</li>
 *      <li><code>mmmm</code> - Month name (long form)</li>
 *      <li><code>dd&nbsp;&nbsp;</code> - Day of month</li>
 *      <li><code>ddd&nbsp;</code> - Week Day (sort form)</li>
 *      <li><code>dddd</code> - Week Day (Long form)</li>
 *      <li><code>hh&nbsp;&nbsp;</code> - Hour</li>
 *      <li><code>nn&nbsp;&nbsp;</code> - Minute</li>
 *      <li><code>ss&nbsp;&nbsp;</code> - Second</li>
 *      <li><code>ss&nbsp;&nbsp;</code> - Second</li>
 *      <li><code>Integer</code> - The value is an Integer</li>
 *      <li><code>Parse</code> - The value is parsed with <code>Date.parse()</code> method.</li>
 *      <li><code>today</code> - The current date according to the browser's clock.</li>
 *    </ul>
 *
 * <br><p><b>The <code>today</code> reserved word</b><p>
 * If <code>today</code> reserved word is used as a date value it is substituted with
 * the current date according to the browser's clock.
 *
 * <br><p><b>The Relative Date Definitions</b><p>
 * The date values can be defined relatively by using relative date definition symbols after the date values.
 * <p><u>Examples:</u><br>
 * <table border="1" style="border-collapse: collapse" cellspacing="3">
 *   <tr><td><code>2007.01.01 <b>+1y</b></code></td><td>One year ago from the date</td></tr>
 *   <tr><td><code>today <b>-1m</b></code></td><td>One month ago from today</td></tr>
 *   <tr><td><code>today <b>+10d</b></code></td><td>10 days later</td></tr>
 * </table>
 * <p>&nbsp;
 *
 */
function DateManager() {

  /**
   * Informs the DateBox object that there is a change in the date.
   * @param node The DOM node of the HTML INPUT Element that has changed
   */
  this.dateChanged = function(node) {

    var f = OMgr.getObject(node) || OMgr.getEnclosingObject(node);
    if (f.changed) {
      f.changed();
    }

  };

  /**
   * Informs the DataDialog object about the change in the Year.
   * @param node The DOM node of the Year element node that has changed
   */
  this.yearChanged = function(node) {
    var f = OMgr.getEnclosingObject(node);
    if (f) f.yearChanged(true);
  };

  /**
   * Informs the DataDialog object about the change in the Month.
   * @param node The DOM node of the Month element node that has changed
   */
  this.monthChanged = function(node) {
    var f = OMgr.getEnclosingObject(node);
    if (f) f.monthChanged(true);
  };

  /**
   * Informs the DataDialog object about the change in the Day.
   * @param node The DOM node of the Day element node that has changed
   */
  this.dayChanged = function(node) {
    var f = OMgr.getEnclosingObject(node);
    if (f) f.dayChanged(true);
  };


  this.showCalendar = function(node) {
    var ccNode = OMgr.findParentNode(node,"mo:type","CalendarBox");
   if (ccNode) {
     var cc = OMgr.getObject(ccNode);
     __currCalendarBox = cc;
     cc.show();
   }
  };


  this.calendarSelectHandler = function(cal, date) {
   __currCalendarBox.dateSelectHandler(cal,date);
  };

  this.calendarCloseHandler = function(cal) {
    cal.hide();
    _dynarch_popupCalendar = null;
  };

  this.calendarDateStatusHandler = function(date) {
   if (__currCalendarBox) {
     return __currCalendarBox.dateStatusHandler(date);
   }
  };

  /**
   * Parses the date given in the value string according to the specified format and returns it.
   * @param valueStr The string that keeps the date to parse.  See {@link #dateFormatingSymbols}
   */
  this.parseDate = function(valueStr, formatStr) {

    var theDate = new Date();

    var diffStr = "";

    valueStr += "";
    var i = valueStr.indexOf(" +");
    if (i == -1) i = valueStr.indexOf(" -");

    if (i > -1) {
      diffStr = valueStr.substring(i);
      valueStr = valueStr.substring(0,i);
    }

    if (valueStr == "today") valueStr = this.formatDate( new Date(), formatStr);
    else {
      if (( valueStr != "" && !isNaN(valueStr)) && valueStr.indexOf(".") == -1 && valueStr.length > 11 ) {
        theDate = new Date();
        theDate.setTime(valueStr);
        valueStr = this.formatDate( theDate, formatStr );
      }
    }

    if (formatStr == "Integer") {
      theDate.setTime(valueStr);
      return theDate;
    }

    if (formatStr == "Parse") {
      theDate.setTime(Date.parse(valueStr));
      return theDate;
    }

    var yy = 1900; var mm = 1; var dd = 1; var hh = 0; var nn = 0; var ss = 0;

    try {

     var delimiters = new RegExp("[.:\\, \/\-]");
     var formats = formatStr.split(delimiters);

     if (formats.length < 2) {

       var i = formatStr.indexOf("yyyy");
       if (i > -1) yy = this.parseIfExists(formatStr,"yyyy",valueStr,yy);
       else {
         i = formatStr.indexOf("yy");
         if (i > -1) yy = this.parseIfExists(formatStr,"yy",valueStr,yy);
       }

       mm = this.parseIfExists(formatStr,"mm",valueStr,mm);
       dd = this.parseIfExists(formatStr,"dd",valueStr,dd);
       hh = this.parseIfExists(formatStr,"hh",valueStr,hh);
       nn = this.parseIfExists(formatStr,"nn",valueStr,nn);

       i = formatStr.indexOf("ss");
       if (i > -1) {
         ss = this.parseIfExists(formatStr,"ss",valueStr,ss);
       } else {
         ss = this.parseIfExists(formatStr,"s",valueStr,ss);
       }

     } else {

       var values = valueStr.split(delimiters);
       var len = values.length;
       if (len > formats.length) len = formats.length;

       for (var i = 0; i < len; i++) {
         var f = formats[i];
         var val = this.parseUnitValue(f,values[i]);
         if (f == "yy" || f == "yyyy") yy = val;
         else if (f == "mm" || f == "mmm" || f == "mmmm") mm = val;
         else if (f == "dd") dd = val;
         else if (f == "hh") hh = val;
         else if (f == "nn") nn = val;
         else if (f == "ss" || f == "s") ss = val;
       }

       if (this.nbrOfDays(yy,mm-1) < dd) return "NaN";

     }
    } catch (e) {
      return "NaN";
    }

    diffStr = diffStr.substring(1);
    if (diffStr != "") {
      var o = diffStr.substring(0,1);
      if (o == "+" || o == "-") {
        var l = diffStr.length;
        var t = diffStr.substring(l-1);
        if ("ymdhns".indexOf(t) > -1) {
          var v = diffStr.substring(1,l-1);
          if (v != "" && !isNaN(v)) {
            var s = t + t + "=" + t + t + o + v;
            eval(s);
          }
        }
      }
    }

    theDate = new Date(yy, mm - 1 , dd, hh, nn, ss, 0);
    return theDate;

  }

  this.parseIfExists = function(formatStr,format,valueStr,target) {
    var x = formatStr.indexOf(format);
    if (x > -1) target = this.parseUnitValue(format,valueStr.substring(x,x+format.length));
   return target;
  };

  this.parseUnitValue = function(format,valueStr) {

   var value = 0;
   if (format == "yyyy" || format.length < 3) {
     if (isNaN(valueStr)) throw "Error";
      value = parseInt(valueStr,10);
   }

   if (format == "yyyy") return value;
   if (format == "yy") {
      if (value < 30) value = value + 2000;
      else value = value + 1900;
      return value;
    }

   if (format == "dd") {
     if (value > 31) throw "Error!";

   } else if (format == "mm"){
     if (value < 1 || value > 12) throw "Error!";
   }

   if (format.length < 3) return value;

   if (format == "mmm") return this.parseShortMonth(valueStr);
   if (format == "mmmm") return this.parseLongMonth(valueStr);

   return "NaN";

  };

  this.parseShortDay = function(day) {
    for (var i = 0; i < DateResource.shortDays.length; i++) {
      if (day == DateResource.shortDays[i]) return i;
    }
    throw "Error!";
  };

  this.parseLongDay = function(day) {
    for (var i = 0; i < DateResource.days.length; i++) {
      if (day == DateResource.days[i]) return i;
    }
    throw "Error!";
  };

  this.parseShortMonth = function(month) {
    for (var i = 0; i < DateResource.shortMonths.length; i++) {
      if (month == DateResource.shortMonths[i]) return i+1;
    }
    throw "Error!";
  };

  this.parseLongMonth = function(month) {
    for (var i = 0; i < DateResource.months.length; i++) {
      if (month == DateResource.months[i]) return i+1;
    }
    throw "Error!";
  };


  /**
   * Builds a formatted data string and returns it.
   * @param theDate The data as Date object
   * @param formatStr The format string. See <a href="DateBox.html#dateFormatingSymbols">DateBox.props.valueFormat</a>
   */
  this.formatDate = function(theDate, formatStr) {

    if (formatStr == "Integer")
      return theDate.getTime();

    var r = "";
    var f = "";
    var p = "";

    for (var i = 0; i < formatStr.length; i++) {

      var s = formatStr.substr(i,1);
      if ("dmyhns".indexOf(s) > -1) {
        if (s == p) f = f + s;
        else {
          if (f != "")
            r = r + this.getValueByFormat(theDate, f);
          f = s;
        }
        p = s;

      } else {

        p = "";
        if (f != "") {
          r = r + this.getValueByFormat(theDate, f);
          f = "";
        }
        r = r + s;
      }
    }
    if (f != "")
      r = r + this.getValueByFormat(theDate, f);

    return r;

  };

  /**
   * Converts a date from one format to another and returns it.
   * @param value The string that contains a date in input format
   * @param iformat The input format
   * @param oformat The output format
   */
  this.convertDate = function(value, iformat, oformat) {
    if (value == "") return "";
    return this.formatDate( this.parseDate(value,iformat), oformat );
  };

  this.getValueByFormat = function(theDate, format) {

    if (format == "dddd") return DateResource.days[theDate.getDay()];

    if (format == "ddd") {
      return DateResource.shortDays[theDate.getDay()];
    }
    if (format == "dd") {
      var x = theDate.getDate();
      return (x < 10) ? '0' + x : x;
    }

    if (format == "d") return theDate.getDate();

    if (format == "mmmm") return DateResource.months[theDate.getMonth()];
    if (format == "mmm") {
      return DateResource.shortMonths[theDate.getMonth()];
    }

    if (format == "mm") {
      var x = theDate.getMonth() + 1;
      return (x < 10) ? '0' + x : x;
    }

    if (format == "m") return theDate.getMonth() + 1;

    if (format == "yyyy") return theDate.getFullYear();
    if (format == "yy") {
      var x = theDate.getFullYear() + "";
      return x.substr(2,4);
    }

    if (format == "hh") {
      var x = theDate.getHours();
      return (x < 10) ? '0' + x : x;
    }
    if (format == "h") return theDate.getHours();

    if (format == "nn") {
      var x = theDate.getMinutes();
      return (x < 10) ? '0' + x : x;
    }

    if (format == "n") return theDate.getMinutes();

    if (format == "ss") {
      x = theDate.getSeconds();
      return (x < 10) ? '0' + x : x;
    }

    if (format == "s") return theDate.getSeconds();

    return "??";

  };

  /** @private */
  this.nbrOfDays = function(year, month) {
    var result = 31;
    if (month == 3 || month == 5 || month == 8 || month == 10) result = 30;
    if (month == 1 && (year/4) != Math.floor(year/4)) result = 28;
    if (month == 1 && (year/4) == Math.floor(year/4)) result = 29;
    return result;
  };

  /** @private */
  this.isWeekendDay = function(year, month, day) {
    var theDate;
    if (year.getTime) theDate = year;
    else theDate = new Date(year, month, day);
    var theDay = theDate.getDay();
    return theDay == 0 || theDay == 6;
  };

};


/**
 * Global Singleton DateManager object
 * @type DateManager
 */
var DateMgr = new DateManager();