/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the 
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains OptionBox class.
 * @author Erkan Cengiz
 * @version $Id: OptionBox.js 22 2008-03-06 15:36:13Z erkan $
 */

/**
 * @class
 * This class wraps a container element that includes two or more HTML Input ("radio") Elements
 * and manages it as an input field. The container element can be any HTML DOM element that can contain
 * form input elements. i.e. <CODE>DIV</CODE> or <CODE>TABLE</CODE>
 * <PRE><CODE>
 * &lt;div
 *   mo:type="OptionBox" mo:field="true" mo:name="GENDER" mo:needsInit="true"
 *   mo:props="nonBlank:true, warnColor:'Red'" class="c3" mo:iValue="F" &gt;
 *
 *   &lt;input type="radio" value="M" onclick="PMgr.selectOption(this)" name="_x"/&gt;
 *   &lt;input type="radio" value="F" onclick="PMgr.selectOption(this)" name="_x"/&gt;
 *   &lt;input type="hidden" name="pf_GENDER"&gt;
 *
 * &lt;/div&gt; </CODE></PRE> The container element should contain a <CODE>INPUT type="hidden"</CODE> element
 * whose name attribute has a prefix <CODE>"pf_"</CODE> to indicate that it is a Process Field.
 * The values of the name attributes of the <CODE>INPUT type="radio"</CODE> elements must be identical so that
 * they act as a group.
 * @extends ProcessField
 */
function OptionBox(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isOptionBox = true;

  /** Array that keeps the references to the wrapped HTML Input ("radio") Elements
   * @type Array
   */
  this.options = null;

  /**
   * The HTMLElement object of <code>HIDDEN TEXT</code> element that keeps the
   * value to submit to the server.
   * @type HTMLElement
   */
  this.selm = null; //-<

  /** @private */
  this.wrapElements = function() {
    this.options = new Array();
    var elms = this.node.getElementsByTagName("input");
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      if (elm.name.startsWith("__t_")) {
        this.options[this.options.length] = elm;
      }
      else if (elm.name.startsWith("pf_")) {
        this.selm = elm;
      }
    }
  };

  /** Sets the value of this field object and checks the
   * radio button whose value is equal to the specified value.
   * @param value The value to set. It must be equal to the value of one of the radio buttons wrapped by this
   * field object.
   */
  this.setValue = function(value) {
    if (value == null) return;
    if (this.selm) {
      this.selm.value = value;
    }
    for (var i = 0; i < this.options.length; i++) {
      var o = this.options[i];
      if (o && o.value == value) {
        o.checked = true;
        break;
      }
    }
  };

  /** Returns the value of this field object
   * @type String
   */
  this.getValue = function() {
    return this.selm.value;
  };


  this.setRecordNumber = function(recnum) {
    var htmlid = "__t_" + OMgr.getUniqueId("");
    for (var i = 0; i < this.options.length; i++) {
      var o = this.options[i];
      if (o) {
        o.id = htmlid;
      }
    }
  };

  /**
   * Clears the checked option and sets the value to blank
   */
  this.clear = function() {
    this.selm.value = "";
    for (var i = 0; i < this.options.length; i++) {
      var o = this.options[i];
      o.checked = false;
    }
  };

  /**
   * Sets the value of this field object to the value of the selected radio button.
   * This method should be called by the onclick event procedures of all the radio button elements.
   */
  this.selectOption = function() {
    for (var i = 0; i < this.options.length; i++) {
      var o = this.options[i];
      if (o && o.checked) {
        this.selm.value = o.value;
        this.setModified();
        break;
      }
    }
  };

  this.wrapElements();

  this.init = function() {
    this.setValue(this.getIValue());
  };

  /** Validates this field object */
  this.validate = function() {
    if (this.boolProp("nonBlank") && this._validate() == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
  };

  /** @private */
  this.getAnOptionElement = function() {
    if (this._anOptionElement) return this._anOptionElement;
    for (var i = 0; i < this.options.length; i++) {
      var o = this.options[i];
      if (o) {
        this._anOptionElement = o;
        return o;
      }
    }
    return null;
  };

  /** @private */
  this._validate = function(handler) {

    for (var i = 0; i < this.options.length; i++) {
      var o = this.options[i];
      if (o && o.checked) return true;
    }

    var covered = false;
    var msg = "CANNOT_BE_BLANK";
    if (this.valErrorHandler) {
      covered = this.valErrorHandler.run(this,msg);
    }

    if (covered == false) {
      alert(OMgr.resource(msg));
      if (this.props.warnColor) this.node.style.backgroundColor = this.props.warnColor;
      return false;
    }

    return true;
  };

};

/** @ignore */
function _ignore() {
  new OptionBox();
}