/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.adom;

import java.util.Properties;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.moremotion.util.StringParser;

/**
 * Represents a single node in ADOM (Application Data Object Model).
 * <P>Each node has a name, a value which can be set and used as <CODE>String, Integer, Double</CODE> and <CODE>boolean</CODE>.
 * Each node can have a set of attributes and a set of child nodes.</P>
 * <P>Since ADOMs and ADOMNodes are used in XML content generation,
 * a node corresponds to an XML tag. Considering this, the node names should
 * accord to the rules of naming XML tags.</P>
 * @see <A href="package-summary.html#nodepaths_to_access_nodes">Using node paths to access nodes</A>
 * @author Yusuf Boyacgil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMNode.java 51 2008-03-10 11:46:54Z erkan $
 */ 
public class ADOMNode implements java.lang.Cloneable {

  public static final byte
  DT_STRING     =  0, // 'hello'
  DT_NUMBER     =  1, // 123.45
  DT_BOOL       =  2, // true, false
  DT_INTEGER    =  8; // 234

  protected String name = null;

  private Object value = null;

  private Properties attributes = null;
  private LinkedList children = null;
  private ADOMNode parent = null;

  private static final Iterator EMPTY_ITERATOR = new LinkedList().iterator();
  private static final ADOMNode[] EMPTY_ARRAY = new ADOMNode[0];
  private static final boolean PRESERVE_WHITE_SPACE = true;


  public Object clone() throws CloneNotSupportedException {
    ADOMNode copy = (ADOMNode)super.clone();

    if (children != null) {
      copy.children = new LinkedList();
      Iterator itr = children.iterator();
      while (itr.hasNext()) {
        ADOMNode n = (ADOMNode)itr.next();
        copy.children.addLast( n.clone() );
      }
    }
    return copy;
  }

  /**
   * Creates a new ADOMNode and names it.
   *
   * @param name Node name.
   */
  public ADOMNode(String name) {
    this.name = name;
  }

  /**
   * Creates a new ADOMNode, names it and sets its value.
   *
   * @param name Node name.
   * @param value Node value.
   */
  public ADOMNode(String name, String value) {
    this(name);
    setValue(value);
  }

  /**
   * Returns the name of this node.
   *
   * @return Node name.
   */
  public String getName() {
    return this.name;
  }

  /**
   * Sets the name of this node.
   *
   * @param value New name of this node.
   */
  public void setName(String value) {
    this.name = value;
  }

  /*
  public String getValue() {
    return this.value;
  }
   */

  /**
   * Returns the value of this node as String.
   *
   * @return Node value. Note that the node values can be null.
   */
  public String getValue() {
    return getValueAsString();
  }

  /**
   * Returns the value of this node to build XML content.
   * <P>Note that some nodes can contain secret information
   * which should not be included in XML in clear format.
   *
   * @return Either node's value or <CODE>{HIDDEN_VALUE}</CODE> if the value of this node is a HiddenString
   */
  public String getValueForXML() {

    if (this.value instanceof HiddenString) {
      return "{HIDDEN_VALUE}";
    }

    return getValue();
  }

  /**
   * Sets value object of this node
   *
   * @param value Value to be set.
   */
  public void setValue(Object value) {
    this.value = value;
  }

  /**
   * Sets value of this node as <CODE>double</CODE>.
   *
   * @param value <CODE>double</CODE> value.
   */
  public void setValue(double value) {
    this.value = new Double(value);
  }


  /**
   * Sets value of this node as <CODE>Integer</CODE> object
   *
   * @param value <CODE>int</CODE> value.
   */
  public void setValue(int value) {
    this.value = new Integer(value);
  }

  /**
   * Sets value of this node as <CODE>Boolean</CODE> object
   *
   * @param value <CODE>boolean</CODE> value.
   */
  public void setValue(boolean value) {
    this.value = new Boolean(value);
  }


  /**
   * Returns the value of this node as <CODE>Object</CODE>.
   */
  public Object getValueAsObject(){
    return value;
  }

  /**
   * Return the value of the node as <CODE>String</CODE>.
   */
  public String getValueAsString(){
    if (value == null) return null;

    if (this.value instanceof Double) {
      return org.moremotion.evaluator.EvalUtils.toStr(this.value);
    }

    return value.toString();
  }

  /**
   * Return the value of the node as <CODE>int</CODE>. If the value of the node cannot be converted
   * to integer '0' is returned.
   */
  public int getValueAsInt() {

    if (value == null) return 0;

    if (value instanceof Integer) {
      return ((Integer)value).intValue();
    }
    if (value instanceof Boolean) {
      return ((Boolean)value).booleanValue() ? 1 : 0;
    }

    try {
      return Integer.parseInt(value.toString());
    } catch (Exception ex){
      return 0;
    }

  }


  /**
   * Return the value of the node as <CODE>double</CODE>. If the value of the node cannot be converted
   * to a number, '0' is returned.
   */
  public double getValueAsDouble() {

    if (value == null) return 0;

    if (value instanceof Double) {
      return ((Double)value).doubleValue();
    }

    if (value instanceof String || value instanceof HiddenString) {
      try {
        return Double.parseDouble(value.toString());
      } catch (Exception ex) { return 0; }
    }

    if (value instanceof Boolean) {
      return ((Boolean)value).booleanValue() ? 1 : 0;
    }

    return 0;
  }

  /**
   * Returns the value of this node as <CODE>boolean</CODE>. Returns <CODE>false</CODE> if the value of the node cannot be converted
   * to boolean.
   */
  public boolean getValueAsBoolean() {

    if (value == null) return false;

    if (value instanceof Boolean) {
      return ((Boolean)value).booleanValue();
    }

    if (value instanceof String  || value instanceof HiddenString) {
      return value.toString().equals("true");
    }

    if (value instanceof Double) {
      return ((Double)value).doubleValue() > 0;
    }

    return false;
  }

//  private void _setValue(String value){
//    if (value == null) return;
//
//    setValueAsString(value);
//  }

  /**
   * Returns <CODE>true</CODE> if this node is the root node of the ADOM, <CODE>false</CODE> otherwise.
   */
  public boolean isRoot() {
    return this.parent == null;
  }

  /**
   * Returns the parent node of this node.
   *
   * @return <CODE>null</CODE> if this node is the root node, otherwise the parent node.
   */
  public ADOMNode getParent() {
    return this.parent;
  }

  /**
   * Sets the parent node of this node.
   */
  public void setParent(ADOMNode value) {
    this.parent = value;
  }

  /**
   * Returns the root node
   */
  public ADOMNode getRootNode(){
    if (parent == null) return this;
    return parent.getRootNode();
  }

  private void getPath(ADOMNode node, StringBuffer sb) {
    if (node.parent != null) {
      getPath(node.parent, sb);
      sb.append("/");
      sb.append(node.name);
    }
  }

  /**
   * Returns the path of this node starting form the root node.
   */
  public String getPath() {  
    if (parent == null) return "/";
    StringBuffer sb = new StringBuffer(25);
    getPath(this,sb);
    return sb.toString();
  }

  ADOMNode getFirstChild(String name){
    if (children == null) return null;
    Iterator it = children.iterator();
    while (it.hasNext()){
      ADOMNode node = (ADOMNode) it.next();
      if (node.getName().equals(name)) {
        return node;
      }
    }
    return null;
  }


  /**
   * Adds a new node attribute to this node.
   *
   * @param name Name of the attribute
   * @param value Value of the attribute
   *
   * @return <CODE>null</CODE> if the attribute does not already exists, its value Otherwise.
   */
  public String addAttribute(String name, String value) {
    if (this.attributes == null) {
      this.attributes = new Properties();
    }
    return (String) this.attributes.setProperty(name, value);
  }

  /**
   * Removes a node attribute from this node by its name.
   * @param name Name of the attribute.
   * @return Value of the removed attribute. If no attribute is found by the name
   * given then <CODE>null</CODE> value is returned.
   */
  public String removeAttribute(String name) {
    if (this.attributes == null) {
      return null;
    }
    return (String) this.attributes.remove(name);
  }

  /**
   * Returns the value of a node attribute.
   *
   * @param name Name of the attribute.
   * @return Value of the attribute, <CODE>null</CODE> if attribute does not exist.
   */
  public String getAttribute(String name) {
    if (this.attributes == null) {
      return null;
    }
    return this.attributes.getProperty(name);
  }

  /**
   * Returns the attributes of this node as a <CODE>Properties</CODE> object.
   */
  public Properties getAttributes() {
    return this.attributes;
  }

  /**
   * Returns the number of attributes that this node has.
   */
  public int getAttributeCount() {
    if (this.attributes == null) {
      return 0;
    }
    return this.attributes.size();
  }

  /**
   * Adds the given node under this node.
   */
  public void addNode(ADOMNode node) {
    if (this.children == null) this.children = new LinkedList();
    node.setParent(this);
    this.children.addLast(node);
  }

  /**
   * Removes the given child node.
   */
  public boolean removeNode(ADOMNode node) {
    if (this.children == null) {
      return false;
    }

    //original
    return this.children.remove(node);
    //    boolean result = this.children.remove(node);
    //    if (result) {
    //      Index.remove(node);
    //    }
    //    return result;

  }

  public void removeLast() {
    if (this.children != null)
      this.children.removeLast();
  }

  /**
   * Returns an <CODE>Iterator</CODE> interface for the first level child nodes of this node.
   */
  public Iterator getNodes() {
    if (this.children == null) {
      return EMPTY_ITERATOR;
    }
    return this.children.iterator();
  }

  /**
   * Returns an <CODE>ADOMNode</CODE> array from the first level child nodes of this node. If this node has no
   * children then returns an empty array (zero-length).
   */
  public ADOMNode[] getNodesArray() {
    if (this.children == null) {
      return EMPTY_ARRAY;
    }
    ADOMNode[] result = new ADOMNode[this.children.size()];
    this.children.toArray(result);
    return result;
  }

  /**
   * Finds the node by the given node path and returns it.
   * <P><B>Note that</B> the nodes that do not exist in the node path are created</P>
   *
   * @return The found existing node or the created node.
   */
  public ADOMNode getNode(String nodePath) {
    int x = nodePath.indexOf('/');
    if (x > -1) nodePath = nodePath.replace('/','.');
    x = nodePath.indexOf('.');
    if (x == -1) {
      return _getNode(nodePath);
    }
    StringParser nodes = new StringParser( nodePath, '.');
    return getNodeRecurse(nodes);
  }

  private ADOMNode getNodeRecurse(StringParser nodes) {
    ADOMNode result = null;
    result = _getNode(nodes.nextItem());
    if (nodes.hasMore()) {
      result = result.getNodeRecurse(nodes);
    }
    return result;
  }

  private ADOMNode _getNode(String name) {
    if (this.children == null) this.children = new LinkedList();
    boolean found = false;
    Iterator it = this.children.iterator();
    ADOMNode node = null;
    while(it.hasNext()) {
      node = (ADOMNode) it.next();
      if (node.getName().equals(name)) {
        return node;
      }
    }

    ADOMNode _new = new ADOMNode(name);
    this.addNode(_new);
    return _new;
  }


  /**
   * Finds the node by the given node path and returns it.
   *
   * @return The node found or <CODE>null</CODE> if the node is not found.
   */
  public ADOMNode findNode(String nodePath) {

    ADOMNode result = null;
    int x = nodePath.indexOf('/');
    if (x > -1) nodePath = nodePath.replace('/','.');
    x = nodePath.indexOf('.');
    if (x == -1) {
      return _findNode(nodePath);
    }
    StringParser nodes = new StringParser( nodePath, '.');
    return findNodeRecurse(nodes);

  }


  private ADOMNode findNodeRecurse(StringParser nodes) {
    ADOMNode result = null;
    result = _findNode(nodes.nextItem());
    if (result != null && nodes.hasMore()) {
      result = result.findNodeRecurse(nodes);
    }
    return result;
  }

  private ADOMNode _findNode(String name) {

    if (this.children == null) return null;

    Iterator it = this.children.iterator();
    ADOMNode node = null;
    while(it.hasNext()) {
      node = (ADOMNode) it.next();
      if (node.getName().equals(name)) {
        return node;
      }
    }
    return null;
  }

  /**
   * Checks if this node has a child node with the given node name.
   *
   * @param nodeName Name of the child.
   * @return <CODE>true</CODE> if the child node exists, <CODE>false</CODE> otherwise.
   */
  public boolean hasChild(String nodeName) {
    boolean found = false;
    Iterator it = getNodes();
    ADOMNode node = null;
    while(it.hasNext()) {
      node = (ADOMNode) it.next();
      if (node.getName().equals(nodeName)) {
        found = true;
        break;
      }
    }
    return found;
  }

//  /**
//   * Checks if the node specified with the node path exists.
//   *
//   * @return <CODE>true</CODE> if the node exists, <CODE>false</CODE> otherwise.
//   */
//  public boolean existsx(String nodePath) {
//    String[] nodeNames = nodeNames(nodePath);
//    if (nodeNames.length > 0) {
//      boolean childFound = hasChild(nodeNames[0]);
//      if (childFound) {
//        String newNodePath = "";
//        for (int i = 1; i < nodeNames.length ; i++) {
//          newNodePath += nodeNames[i] + ".";
//        }
//        if (!newNodePath.equals("")) {
//          return getNode(nodeNames[0]).existsx(newNodePath);
//        } else {
//          return childFound;
//        }
//      }
//    }
//    return false;
//  }


  public ADOMNode getSubNode(String name) {

    if (this.children == null) return null;

    Iterator it = getNodes();
    ADOMNode node = null;
    while(it.hasNext()) {
      node = (ADOMNode) it.next();
      if (node.getName().equals(name)) {
        return node;
      }
    }
    return null;
  }


  /**
   * Finds the node by the given node path and returns its value. If no node is found then returns an empty string.
   *
   */
  public String getNodeValue(String nodePath) {

    ADOMNode node = findNode(nodePath);
    if (node == null) {
      return "";
    }
    String result = node.getValue();
    if (result == null) {
      return "";
    }
    return result;
  }

  /**
   * Returns the values of the first level child nodes in a <CODE>Properties</CODE> object.
   */
  public Properties getNodeValues() {
    Properties props = new Properties();
    Iterator it = getNodes();
    while(it.hasNext()) {
      ADOMNode child = (ADOMNode) it.next();
      String val = "";
      if ( child.getValue() != null ) {
        val = child.getValue();
      }
      props.setProperty(child.getName(), val);
    }
    return props;
  }

  /**
   * Finds the node by the given node path and sets its value.
   * <P><B>Note that</B> the nodes that do not exist in the node path are created.</P>
   * @param nodePath Path name.
   * @param value Value to set.
   * @return ADOMNode that this method has modified.
   */
  public ADOMNode setNodeValue(String nodePath, String value) {
    ADOMNode node = getNode(nodePath);
    node.setValue(value);
    return node;
  }


  /**
   * Sets the values of the child nodes using a <CODE>Properties</CODE> object.
   * The keys of the Properties object are used as node paths which define the nodes to be modified and the values
   * are used as the values of the nodes.
   *
   * @param props Properties object that contains Node path/Value pairs.
   */
  public void setNodeValues(Properties props) {
    if (props == null) return;
    Iterator it = props.keySet().iterator();
    while(it.hasNext()) {
      String key = (String) it.next();
      String n = new String(key);

      ADOMNode child = getNode(n);
      if (child != null) {
        String val = props.getProperty(key);
        child.setValue(val);
      }
    }
  }

  /**
   * Returns the number of the child nodes of this node.
   */
  public int getNodeCount() {
    if (this.children == null) {
      return 0;
    }
    return this.children.size();
  }

  /**
   * Removes all the child nodes of this node
   */
  public void clear() {
    Iterator it = getNodes();
    while(it.hasNext()) {
      ADOMNode node = (ADOMNode) it.next();
      node.clear();
      it.remove();
    }
  }

  /**
   * Returns the string representation of this node.
   */
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("[ name: " + this.name);
    buffer.append(", value: " + this.value);
    buffer.append(", attributes: " + this.attributes);
    buffer.append(", children (size): " + getNodeCount());
    buffer.append(", parent: " + this.parent);
    buffer.append(" ]");

    return buffer.toString();
  }

  /**
   * Checks the equality of this node to another node with all their content.
   */
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (obj instanceof ADOMNode) {
      ADOMNode another = (ADOMNode) obj;

      boolean namesAreEqual =
      this.name != null && another.getName() != null &&
      this.name.equals(another.getName());

      boolean valuesAreEqual =
      (this.value == null && another.value == null) ||
      (this.value != null && another.getValue() != null &&
      this.getValueAsString().equals(another.getValueAsString()));

      boolean nodeCountsAreEqual =
      this.getNodeCount() == another.getNodeCount();

      boolean attributeCountsAreEqual =
      this.getAttributeCount() == another.getAttributeCount();

      if (namesAreEqual
      && valuesAreEqual
      && nodeCountsAreEqual
      && attributeCountsAreEqual
      ) {

        // first check attributes
        if (this.getAttributeCount() != 0) {
          Properties anotherAttrs = another.getAttributes();
          boolean attributesAreEqual =
          this.attributes != null && anotherAttrs != null &&
          this.attributes.equals(anotherAttrs);

          if (!attributesAreEqual) {
            return false;
          }
        }

        // then check subnodes
        if (this.getNodeCount() != 0) {
          Iterator it = this.children.iterator();
          while(it.hasNext()) {
            ADOMNode node = (ADOMNode) it.next();

            boolean foundAndEquals = false;

            Iterator it2 = another.getNodes();
            while(it2.hasNext()) {
              ADOMNode anotherNode = (ADOMNode) it2.next();
              if ( node.equals(anotherNode) ) {
                foundAndEquals = true;
              }
            }

            if (!foundAndEquals) {
              return false;
            }

          }
        }

        return true;
      }

    }
    return false;
  }


  public void sort(String sortDefinition) {

    if (children == null || sortDefinition.trim().length() == 0) return;

    SortComparator scomp = new SortComparator(sortDefinition);

    java.util.Collections.sort(children, scomp);


  }


  /**
   * Converts this node and all its child nodes to XML string.
   */
  public String toXml() {
    return _toXml(null, -1);
  }
  /**
   * Converts this node and all its child nodes to XML string.
   *
   * @param tagName The name of the XML tag.
   * If parameter is null then no tag will be included in the XML.
   */
  public String toXml(String tagName) {
    if (tagName == null || tagName.length() == 0) {
      return _toXml("", -1);
    }
    return _toXml(null, -1);
  }

  /**
   * Converts this node and all its child nodes to XML string. The XML elements in the returned string are
   * indented with new line and tab characters.
   */
  public String toXmlWithIndent() {
    return _toXml(null, 0);
  }

  private String _toXml(String tagName, int indent) {

    //    System.out.println("tagName : " + tagName + ", indent :" + indent);
    StringBuffer buffer = new StringBuffer();

    if (indent > -1 && tagName == null) {
      buffer.append('\n');
      for (int i = 0; i < indent; i++ ) {
        buffer.append("  ");
      }
    }

    String tname = null;
    if (tagName == null) {
      tname = this.name;
    } else if (tagName.length() > 0 ) {
      tname = tagName;
    }

    if (tname != null) {
      buffer.append("<" + tname);
      // attributes
      if (attributes != null) {
        Iterator it = attributes.keySet().iterator();
        while(it.hasNext()) {
          buffer.append(" ");
          String attrname = (String) it.next();
          buffer.append(attrname);
          buffer.append("=\"");
          buffer.append( org.moremotion.page.PageUtils.encodeXMLAttributeValue( attributes.getProperty(attrname) ) );
          buffer.append("\"");
        }
      }
    }
    if ((value == null || value.equals("")) && (children == null)) {
      if (tname != null) buffer.append("/>");
    } else {
      if (tname != null) buffer.append(">");
      if (value != null && !value.equals("")) {
        buffer.append( org.moremotion.page.PageUtils.encodeXMLElementValue(value.toString())  );
      }
      if (children != null) {
        Iterator it = children.iterator();
        while (it.hasNext()) {
          ADOMNode child = (ADOMNode)it.next();
          buffer.append(child._toXml(null,indent > -1 ? indent+1 : -1));
        }
      }
      if (tname != null) {
        if (children != null && indent > -1 && tagName == null) {
          buffer.append('\n');
          for (int i = 0; i < indent; i++ ) {
            buffer.append("  ");
          }
        }
        buffer.append("</" + tname + ">");
      }
    }
    return buffer.toString();
  }

  //
  // Private Territory
  //
  private String normalize(String nodePath) {
    if (nodePath == null) {
      return nodePath;
    }
    String result = new String(nodePath.trim());
    if (result.startsWith("/")) {
      result = result.substring(1);
    }
    return result.replace('/', '.');
  }

  private String[] nodeNames(String nodePath) {
    nodePath = normalize(nodePath);
    LinkedList nodeList = new LinkedList();
    StringTokenizer st = new StringTokenizer(nodePath, ".");
    while(st.hasMoreTokens()) {
      nodeList.addLast(st.nextToken());
    }
    String[] result = new String[nodeList.size()];
    nodeList.toArray(result);
    return result;
  }


  /** Test method.
   */
  public static void main(String[] args) throws Exception {

    testGetNode();

  }




  static void testGetNode() throws Exception {

    ADOMNode root = new ADOMNode("root");
    root.setNodeValue("aaa","aaa value");
    root.setNodeValue("aaa.bbb","bbb value");

    System.out.println( "aaa ---> " + root.findNode("aaa") );
    System.out.println( "bbb ---> " + root.findNode("aaa.bbb") );

    System.out.println(root.toXmlWithIndent());



  }


  static void testSort() throws Exception {

    ADOMNode root = new ADOMNode("root");
    ADOMNode item;

    item = new ADOMNode("item");
    item.setNodeValue("NAME","AAA");
    item.setNodeValue("PRICE","12.34");
    root.addNode(item);

    item = new ADOMNode("item");
    item.setNodeValue("NAME","AAA");
    item.setNodeValue("PRICE","12.36");
    root.addNode(item);

    item = new ADOMNode("item");
    item.setNodeValue("NAME","AAA");
    item.setNodeValue("PRICE","48.2");
    root.addNode(item);

    item = new ADOMNode("item");
    item.setNodeValue("NAME","BBB");
    item.setNodeValue("PRICE","0.34");
    root.addNode(item);

    item = new ADOMNode("item");
    item.setNodeValue("NAME","BBB");
    item.setNodeValue("PRICE","22");
    root.addNode(item);

    root.sort("NAME,d,false;PRICE,d,1");

    Iterator itr = root.getNodes();
    while (itr.hasNext()) {
      item = (ADOMNode)itr.next();
      System.out.println(item.getNodeValue("NAME") + "  -  " + item.getNodeValue("PRICE") );
    }



  }

}