/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * CustomLanguage.java
 *
 * Created on 15 September 2005 Saturday, 11:23
 */

package org.moremotion.clang;

import java.util.ArrayList;
import org.moremotion.evaluator.ADOMName;
import org.moremotion.evaluator.Constant;
import org.moremotion.evaluator.Expression;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.evaluator.RemoveInstructionException;
import org.moremotion.evaluator.Symbol;
import org.moremotion.evaluator.EvalUtils;
import org.moremotion.process.ProcessRecord;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;


/**
 * Base class for developing custom languages. The classes that extends this base class should implement getSyntax()
 * and excuteFunction() methods.
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CustomLanguage.java 63 2008-03-14 16:29:03Z erkan $
 */
abstract public class CustomLanguage {
  
  //  private char dlm = '\n';
  private char dlm = ' ';
  private LanguageSyntax lang;
  private static final String DLM = "'\"+-%/(),;* =";
  protected MMSymbolResolver sr;
  protected MoreMotionRequest request;
  protected Logger logger = null;
  private ProcessRecord prec = null;
  
  abstract public LanguageSyntax getSyntax();
  abstract public int executeFunction(int funcid, Object[] args) throws CLException;
  
  
  public CustomLanguage(MoreMotionRequest request, Logger parent) {
    this.request = request;
    this.sr = request.getSymbolResolver();
    this.lang = getSyntax();
    
    this.logger = parent;
  }
  
  public int execute(String s) throws CLException {
    int maxcc = 0;
    try  {
      MScriptResolver resolver = new MScriptResolver(s, sr, dlm);
      String line = resolver.nextStatement();
      while (line != null && maxcc <= 4) {
        if (logger.LEVEL2) logger.printres("SML_STATEMENT",line);
        int cc = _execute(line);
        if (cc > maxcc) maxcc = cc;
        if (logger.LEVEL3) logger.printres("SML_STATEMENT_EXECUTED", ""+cc);
        line = resolver.nextStatement();
      }
    } catch (RemoveInstructionException rie) {
      return 0;
    } catch (Exception e) {
      throw new CLException(e);
    }
    return maxcc;
  }
  
  public int _execute(String s) throws CLException, RemoveInstructionException {
    
    int cc = -1;
    
    s = s.trim();
    if (s.length() == 0) return 0;
    boolean found = false;
    
    for (int i = 0; i < lang.functions.length && cc < 0; i++) {
      String func = lang.functions[i].name + "(";
      if (s.startsWith(func)) {
        found = true;
        int k = StringParser.locateChar(s, func.length(),')');
        if (k > -1) {
          String parmsstr = s.substring(func.length(),k);
          Object[] parms;
          try {
            parms = getParameters(parmsstr,i);
            if (logger.LEVEL2) {
              debugParameters(parms,i);
            }
            cc = executeFunction(lang.functions[i].id,parms);
          } catch (RemoveInstructionException rie) {
            throw rie;
          } catch (Exception e) {
            throw new CLException("CL_ERROR",new String[]{lang.name,s},e);
          }
        } else {
          throw new CLException("CL_ERROR_MISSING_CLOSING_PARANTHESIS",new String[]{lang.name, func + ")"},null);
        }
      } 
    }
    
    if (!found) {
      throw new CLException("CL_ERROR_UNKNOWN_FUNCTION",new String[]{lang.name, s},null);
    }    
    
    return cc;
  }
  
  public static boolean isConstant(String s) {
    if (s.length() == 0) return false;
    if (s.equalsIgnoreCase("true") || s.equalsIgnoreCase("false")) {
      return true;
    }
    try {
      Double.parseDouble(s);
      return true;
    } catch (Exception ignore){}
    
    if (s.charAt(0) == '\'') {
      int i = StringParser.locateChar(s, 0, '\'');
      if (i == -1) return false;
      if (i+1 != s.length()) return false;
      return true;
    }
    return false;
  }
  
  private String von(Object o) {
    if (o == null) {
      return "{null}";
    }
    return o.toString();
  }
  
  private void debugParameters(Object[] parms, int fi) {
    ParameterSyntax[] pss = lang.functions[fi].parameters;
    logger.printres("SML_FUNCTION", lang.functions[fi].name);
    for (int i = 0; i < pss.length; i++ ) {
      ParameterSyntax ps = pss[i];
      Object o = parms[i];
      if (o != null && ps.type == ParameterSyntax.PT_PASSWORD) {
        logger.print("  " + fixed(ps.name) + " : ***");
      } else if (o != null && (ps.type == ParameterSyntax.PT_LIST || ps.type == ParameterSyntax.PT_RESOLVED_LIST )) {
        logger.print("  " + fixed(ps.name) + " : " + debugList((Object[])o)); 
      } else {
        logger.print("  " + fixed(ps.name) + " : " + von(o));
      }
    }
    
  }
  
  private String debugList(Object[] sl) {
    StringBuffer sb = new StringBuffer();
    sb.append("(");
    if (sl != null) {
      for (int i = 0; i < sl.length; i++) {
        if (i > 0) sb.append(", ");
        sb.append(XString.create(sl[i].toString()).doubleTheQuotes().encloseWithQuotes().s());
      }
    }
    sb.append(")");
    return sb.toString();
  }
  
  private Object[] getParameters(String str, int fi) throws CLException, RemoveInstructionException {
    
    ArrayList a = new ArrayList(8);
    str = str.replace('\n',' ');
    StringParser sp = new StringParser(str,',');
    ParameterSyntax[] pss = lang.functions[fi].parameters;
    for (int i = 0; i < pss.length; i++ ) {
      ParameterSyntax ps = pss[i];
      String s = sp.nextItem().trim();
      if (s.length() == 0) {
        if (ps.mandatory) {
          throw new CLException("CL_REQUIRED_PARM_MISSING",new String[]{lang.functions[fi].name,ps.name},null);
        }
        a.add(null);
      } else {
        
        if (ps.type == ParameterSyntax.PT_ADOM) {
          if (!EvalUtils.isADOMName(s)) {
            throw new CLException("CL_INVALID_PARM_TYPE",new String[]{ps.name,s,ParameterSyntax.getTypeText(ps.type)},null);
          }
          a.add(  new ADOMName(s) );
        }  if (ps.type == ParameterSyntax.PT_SYMBOL) {
          if (!EvalUtils.isSymbol(s)) {
            throw new CLException("CL_INVALID_PARM_TYPE",new String[]{ps.name,s,ParameterSyntax.getTypeText(ps.type)},null);
          }
          a.add( new Symbol(s) );
        } else {
          
          try {
            
            switch (ps.type) {
              case ParameterSyntax.PT_STRING : a.add( getValue(s) ); break;
              case ParameterSyntax.PT_PASSWORD : a.add( getValue(s) ); break;
              case ParameterSyntax.PT_INT : a.add(new Integer( getValue(s) )); break;
              case ParameterSyntax.PT_LONG : a.add(new Long( getValue(s) )); break;
              case ParameterSyntax.PT_DOUBLE : a.add(new Double( getValue(s) )); break;
              case ParameterSyntax.PT_BOOLEAN : a.add(new Boolean( getBooleanValue(s) )); break;
              case ParameterSyntax.PT_LIST : a.add(getList(s,false)); break;
              case ParameterSyntax.PT_RESOLVED_LIST : a.add(getList(s,true)); break;
              case ParameterSyntax.PT_TEST_EXPRESSION : a.add(new Expression(s) ); break;
            }
          } catch (RemoveInstructionException rie) {
            throw rie;
          } catch (Exception e) {
            throw new CLException("CL_INVALID_PARM_TYPE",new String[]{ps.name,s,ParameterSyntax.getTypeText(ps.type)},e);
          }
        }
        
        //        if (s.trim().length() == 0 && ps.mandatory) {
        //          throw new CLException("CL_PARM_VALUE_EMPTY",new String[]{lang.functions[fi].name,ps.name},null);
        //        }
        
      }
    }
    
    if (sp.nextItem().trim().length() > 0) {
      throw new CLException("CL_EXCESSIVE_PARAMETERS",new String[]{lang.functions[fi].name},null);
    }
    
    Object[] result = new Object[a.size()];
    a.toArray(result);
    
    return result;
  }
  
  
  
  
  private String getValue(String s) throws Exception {
    // s = replaceHuts(s);
    if (isConstant(s)) {
      s = getConstant(s);
    } else if (EvalUtils.isSymbol(s)) {
      s = sr.getValue(s);
    } else {
      Expression e = new Expression(s);
      s = e.eval(sr);
    }
    return s;
  }
  
  private boolean getBooleanValue(String s) throws Exception {
    
    if (s.equals("true")) return true;
    if (s.equals("false")) return false;
    
    // s = replaceHuts(s);
    if (EvalUtils.looksLikeSymbol(s)) {
      s = sr.getValue(s);
      if (s.equals("true")) return true;
      if (s.equals("false")) return false;
    }
    
    Expression e = new Expression(s);
    return e.evalBool(sr);
    
  }
  
  private static String getValueOfStringConstant(String str, char quote) throws CLException {
    
    if (str == null || str.length() < 2 || str.charAt(0) != quote || str.charAt(str.length()-1) != quote) {
      throw new CLException("INVALID_STRING_CONSTANT");
    }
    
    String result = str.substring(1, str.length()-1);
    
    if (result.indexOf(quote) > -1) {
      StringBuffer buf = new StringBuffer(result.length());
      int i = 0;
      int lengthMinusOne = result.length() - 1;
      while ( i < result.length() ) {
        char c = result.charAt(i);
        buf.append(c);
        if (c == quote) {
          if (i == lengthMinusOne || result.charAt(i+1) != quote) {
            throw new CLException("INVALID_STRING_CONSTANT");
          }
          i = i + 1;
        }
        i++;
      }
      return buf.toString();
    }
    
    return result;
  }
  
  
  private static String getConstant(String str) throws CLException {
    
    if (str == null || str.length() == 0) {
      throw new CLException("INVALID_CONSTANT");
    }
    
    if (str.equalsIgnoreCase("true") || str.equalsIgnoreCase("false")) {
      return str;
    }
    
    if (str.charAt(0) == '\'') {
      return getValueOfStringConstant(str, '\'');
    } else {
      try {
        Double.parseDouble(str);
        return str;
      } catch (Exception ignore){}
    }
    
    throw new CLException("INVALID_CONSTANT");
    
  }
  
  private static int locateHut(StringBuffer buffer, int startPos, char c) {
    int i = startPos;
    if (i > buffer.length()) return -1;
    char quote = ' ';
    for (; i < buffer.length(); i++) {
      char cc = buffer.charAt(i);
      char ch = ' ';
      if (cc == c && quote == ' ') return i;
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      }
      else if (quote == cc) quote = ' ';
      if (quote == ' ') {
      }
      if (ch != ' ') {
        ch = ' ';
      }
      
    }
    return -1;
  }
  
  /*
  private static String replaceHuts(String s) {
    if (s.indexOf('^') == -1) {
      return s;
    }
    StringBuffer ss = new StringBuffer(s);
    int i = locateHut(ss,0,'^');
    if (i == -1) {
      return s;
    }
    while (i > -1) {
      ss.setCharAt(i, '_');
      i = locateHut(ss,i+1,'^');
    }
    return ss.toString();
  }
  */
  
  private Object[] getList(String str, boolean resolve) throws Exception {
    if (str.length() == 0) return null;
    if (str.charAt(0) == '[') str = str.substring(1,str.length()-1);
    StringParser p = new StringParser(str,',');
    p.trimmed();
    ArrayList a = new ArrayList();
    while (!p.eos()) {
      String s = p.nextItem();
      if (resolve) s = getValue(s);
      if (s.equals("*")) a.add( new Symbol(s) ); 
      else if (s.equals("a:*")) a.add( new Symbol(s) ); 
      else if (EvalUtils.looksLikeSymbol(s)) a.add( new Symbol(s) );
      else if (EvalUtils.isADOMName(s)) a.add( new ADOMName(s) );
      else if (EvalUtils.isNumber(s)) a.add( new Constant(new Double(s)) );
      else if (EvalUtils.isBoolean(s)) a.add( new Constant(new Boolean(s)) );
      else if (EvalUtils.isString(s)) a.add( new Constant(  EvalUtils.getValueOfStringConstant(s, '\'') ) );
      else a.add( new Expression(s) );
    }
    Object[] result = new Object[a.size()];
    a.toArray(result);
    return result;
  }
  
  
  private static String fixed(String s) {
    s = s + "                       ";
    return s.substring(0,15);
  }
  
}
