/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ConfigCache.java
 *
 */

package org.moremotion.config;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.moremotion.servlet.SystemContext;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ConfigCache.java 66 2008-03-14 18:06:09Z erkan $
 */
public class ConfigCache {
  
  private static HashMap cache = new HashMap(1024);
  
  /** Creates a new instance of ConfigCache */
  public ConfigCache() {
  }
  
  
  /**
   * Stores the given Configuration Unit in the internal cache. If an unit with the same name already exits
   * it is marked as as unusable before putting the new one onto it. This way the configuration manager classes such as
   * {@link org.moremotion.datasrc.DataSourceManager DataSourceManager} or 
   * {@link org.moremotion.action.ActionServiceManager ActionServiceManager} 
   * can recognise and recompile it.
   */
  public static boolean addUnit(String type, String name, ConfigUnit unit) {
    
    synchronized (cache) {
      
      String key = getKey(type,name);
      
      ConfigUnit oldUnit = (ConfigUnit) cache.get(key);
      if (oldUnit != null) {
        /*
         * Empty the Base Unit and the Data Object of the old unit so that
         * units referring to it will detect the change and relink to a new base unit
         */
        if ((oldUnit.pageName != null && oldUnit.pageName.length() > 0) || unit.pathLength <= oldUnit.pathLength) {
          oldUnit.clear();
        } else {
          SystemContext.logger().printres("CONFIG_UNIT_NOT_LOADED",name);
          return false;
        }
        
      }
      
      cache.put(key, unit);
      return true;
    }
    
  }
  
  /**
   * Returns the ConfigUnit associated with the specified type and name. If the type 
   * identifier is already contained in the name, i.e. <code>"<b>dataSource</b>|products"</code>, the
   * type parameter is ignored.
   * @param type The Unit Type
   * @param name The Unit Name
   */
  public static ConfigUnit getUnit(String type, String name) {
    return (ConfigUnit)cache.get( getKey(type,name)  );
  }
  
  private static String getKey(String type, String name) {
    if (name.indexOf(':') > -1) {
      /*
       * Config type is already contained in the name. So just simply return the name as the result.
       */
      return name;
    }
    
    return type + ":" + name;
  }
  
  
  /**
   * Removes the Configuration Units that are contained in the specified file
   */
  public static void removeIdleUnits(String refId) {
    
    synchronized (cache) {
      Iterator itr = cache.entrySet().iterator();
      while (itr.hasNext()) {
        Map.Entry entry = (Map.Entry)itr.next();
        ConfigUnit unit = (ConfigUnit)entry.getValue();
        if (refId.equals(unit.getRefId())) {
          unit.clear();
          itr.remove();
        }
      }
    }
    
  }
  
  
}
