/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ConfigManager.java
 *
 */

package org.moremotion.config;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.moremotion.util.Logger;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.Logger;
import org.moremotion.util.XString;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * ConfigManager detects the changes in the configuration files under "WEB-INF/MM-INF/config" directory
 * and loads the first level elements in the file as {@link ConfigUnit} objects and lets the {@link ConfigCache}
 * store them.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ConfigManager.java 115 2008-04-08 11:56:01Z erkan $
 */
public class ConfigManager implements Runnable {
  
  private HashMap cfeList = new HashMap();
  private Thread t;
  private boolean stop = false;
  private File root = null;
  private String pagePath = null;
  
  private LinkedList processQueue = null;
  
  
  public static Config getConfig(String type, String name) throws ConfigurationException {
    
    ConfigUnit unit = ConfigCache.getUnit(type, name);
    
    if (unit == null) {
      return null;
    }
    
    if (unit.isNew()) {
      synchronized (unit.getSyncObject()) {
        parse(unit);
      }
    }
    
    return (Config)unit.getDataObject();
  }
  
  public static Config getConfig(String type, String name, String originPage) throws ConfigurationException {
    String cfgname = name;
    if (!cfgname.startsWith("/")) {
      if (originPage != null && originPage.length() > 0) {
        cfgname = originPage + "." + cfgname;
      }
    }
    
    return getConfig(type, cfgname);
  }
  
  private static void parse(ConfigUnit unit) throws ConfigurationException {
    
    ConfigNodeList nodelist = unit.getConfigNodeList();
    
    // Config config = new Config(unit.getName(), nodelist, true);
    Config config = new Config(unit, true);
    unit.setDataObject(config);
    
  }
  
  
  /**
   * Creates a new instance of ConfigManager
   */
  public ConfigManager(File root) {
    this.root = root;
    this.pagePath = (new File(root,"pages")).getAbsolutePath();
    start();
  }
  
  public void start() {
    t = new Thread(this);
    t.setDaemon(true);
    t.start();
  }
  
  private static ConfigManager Instance = null;
  
  public void run() {
    
    while(!stop) {
      
      try {
        
        synchronized (this) {
          
          resetFlags();
          processQueue = new LinkedList();
          
          // File[] files = root.listFiles(new ConfigFilesFilter());
          List files = getFileListing(root);
          if (files != null) {
            for (int i = 0; i < files.size(); i++) {
              File f = (File) files.get(i);
              if (needsProcessing(f)) {
                processQueue.addLast(f);
              }
            }
          }
          
          for (int i = 0; i < processQueue.size(); i++) {
            processFile((File)processQueue.get(i));
          }
          processRemovedFiles();
          
          t.sleep(1000L * SystemConfig.configurationScanIntervalInSeconds());
        }
      } catch(Exception ex) {
        ex.printStackTrace();
      }
    }
    
  }
  
  static public List getFileListing(File startDir) {
    
    ArrayList list = new ArrayList();
    collectFiles(startDir,list);
    return list;
    
  }
  
  static void collectFiles(File dir, ArrayList list) {
    
    if (!dir.exists()) return;
    
    File[] files = dir.listFiles();
    if (files == null || files.length == 0) {
      return;
    }
    
    File f = null;
    for (int i = 0; i < files.length; i++) {
      f = files[i];
      if (f.isFile() && f.getName().endsWith(".xml")) {
        list.add(f);
      }
      if (f.isDirectory()) {
        collectFiles(f,list);
      }
      
    }
    return;    
    
  }
  
  public void stop() {
    this.stop = true;
  }
  
  public void finalize() {
    stop();
  }
  
  private boolean needsProcessing(File f) throws ConfigurationException {
    
    ConfigFileEntry cfe = (ConfigFileEntry)cfeList.get( f.getAbsolutePath() );
    if (cfe != null) {
      cfe.processed = true;
    }
    return cfe == null || cfe.isNewer(f);
    
  }
  
  
  private String getRelativeName(File f) {
    return XString.after(f.getAbsolutePath(),root.getAbsolutePath()+File.separator);
  }
  
  private void processFile(File f) throws ConfigurationException {
    
    String key = f.getAbsolutePath();
    
    ConfigFileEntry cfe = (ConfigFileEntry)cfeList.get(key);
    
    if (cfe == null) {
      SystemContext.logger().printres("LOADING_CONFIG_FILE",getRelativeName(f));
      cfe = new ConfigFileEntry(f);
      cfe.processed = true;
      parseConfigFile(f, cfe.getRefId());
      cfeList.put(key,cfe);
      
    } else {
      
      cfe.processed = true;
      SystemContext.logger().printres("RELOADING_CONFIG_FILE",getRelativeName(f));
      String oldRefId = cfe.getRefId();
      cfe.update(f); // lmodified and refId updated
      parseConfigFile(f, cfe.getRefId());
      ConfigCache.removeIdleUnits(oldRefId);
    }
    
  }
  
  private void resetFlags() {
    Iterator itr = cfeList.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      ConfigFileEntry cfe = (ConfigFileEntry)entry.getValue();
      cfe.processed = false;
    }
  }
  
  private void processRemovedFiles() {
    Iterator itr = cfeList.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      ConfigFileEntry cfe = (ConfigFileEntry)entry.getValue();
      if (!cfe.processed) {
        ConfigCache.removeIdleUnits(cfe.getRefId());
        itr.remove();
      }
    }
  }
  
  private static DocumentBuilderFactory factory = null;
  private static DocumentBuilder builder = null;
  
  private String getPageName(File f) {
    String s = XString.after(f.getAbsolutePath(),pagePath,null);
    return s == null ? "" : XString.beforeLast(s,'.',s).replace('\\','/') + ".";
  }
  
  private void parseConfigFile(File file, String refId) throws ConfigurationException {
    
//    System.out.println("Parse " + file.getName());
    
    boolean pageConfig = file.getAbsolutePath().startsWith(pagePath);
    int pathLength = file.getParentFile().getAbsolutePath().length();
    
    try {
      
      if (builder == null) {
        factory = DocumentBuilderFactory.newInstance();
        builder = factory.newDocumentBuilder();
      }
      
      Document document = builder.parse(file);
      
      NodeList nlist = document.getElementsByTagName("root").item(0).getChildNodes();
      
      for (int i = 0; i < nlist.getLength(); i++) {
        
        Node node = nlist.item(i);
        
        if (node.getNodeType() == Node.ELEMENT_NODE) {
          String tagName = node.getNodeName();
          String name = ((Element)node).getAttribute("name");
          
          if (pageConfig && name.length() > 0 && name.charAt(0) != '/') {
            XString xs = new XString(file.getAbsolutePath());
            name = xs.after(pagePath).beforeLast('.').replace('\\','/').s() + '.' + name;
          }
          
          ConfigUnit unit = new ConfigUnit((Element) node, refId);
          unit.pageName = getPageName(file);
          unit.pathLength = pathLength;
          
          if (tagName.equals("systemParameters")) {
            SystemConfig.refresh(unit,getRelativeName(file).equals("sysinfo.xml"));
          } 
          
//          else if (tagName.equals("component") && name.indexOf(".Security^") > -1) {
//            if ( ((Element)node).getAttribute("isDefault").equals("true") ) {
//              SystemConfig.refresh(unit);
//            }
//          }
          
          else {
            ConfigCache.addUnit(tagName, name, unit);
            
          }
          
        }
      }
      
    } catch (Exception e) {
      throw new ConfigurationException(e);
    }
    
    
  }
  
  /*
  class ConfigFilesFilter implements java.io.FileFilter {
   
    public boolean accept(File file) {
      return file.getName().endsWith(".xml");
    }
   
  }
   *
   **/
  
  private class ConfigFileEntry {
    long lmodified = 0;
    int counter = 0;
    String refIdPrefix;
    String refId;
    boolean processed = false;
    
    ConfigFileEntry(File f) {
//      this.lmodified = f.lastModified();
      this.counter = 0;
      this.refIdPrefix = f.getAbsolutePath().hashCode() + "_";
      update(f);
    }
    
    boolean isNewer(File f) {
      long lm = f.lastModified();
      return f.lastModified() > lmodified;
    }
    
    void update(File f) {
      lmodified = f.lastModified();
      refId = refIdPrefix + ++counter;
    }
    
    String getRefId() {
      return refId;
    }
    
  }
  
  
}
