/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.datasrc;

import java.util.HashMap;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigNode;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.security.AccessControlConfigParameter;
import org.moremotion.security.AccessControlDefinition;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.DOMException;

/**
 * MoreMotion DataSource Configuration
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: DataSourceConfig.java 51 2008-03-10 11:46:54Z erkan $
 */
public class DataSourceConfig extends Config {
  
  private String className = null;
  
  /**
   * Keeps the scope of this DataSource. According to the scope,
   * ADOM, belonging to this datasource, will be stored either in
   * request (if scope is request), or session (if scope is session),
   * or in servlet context (if scope is application).
   * Valid values: 'r', 's' and 'a'.
   */
  
  private char scope = 'r';
  
  /**
   * How many seconds will have to be passed, in order that
   * ADOM related with this DataSourceConfig should be refreshed. If refresh timeout
   * is -1, it means that the related ADOM will not be refreshed.
   * Default value is O (refresh always)
   */
  private ConfigParameter refreshTimeout;
  
  /**
   * Resource bundle name is a reference to a MoreMotion Resource
   * that is used to convert resource messages, i.e. $res(MY_MESSAGE_ID),
   * in the ADOM generated by the datasource.
   */
  private ConfigParameter resbundle = null;
  
  private ConfigParameter unitname = null;
  
  private AccessControlConfigParameter accessControl = null;
  
  private ConfigParameter precond = null;
  
  /**
   * Number of the items to be provided by the data source at a time. A value less than 1 should be interpretted as
   * provide all the items.
   */
  private ConfigParameter itemsPerPage = null;
  
  /**
   * Number of the pages per block. This parameter is used internally by the {@link Paging} object to arrange
   * the total number of pages in blocks.
   */
  private ConfigParameter pagesPerBlock = null;
  
  /**
   * The debugging level from 1 to 3
   */
  private ConfigParameter debug = null;
  
  private boolean languageSensitive = false;
  
  /**
   * Returns <CODE>true</CODE> if the Data Source is language sensitive. A Data Source is marked as
   * Language Sensitive if any of its parameters contain a <CODE>@vof(LANG)</CODE> definition.
   */
  public boolean isLanguageSensitive() {
    return this.languageSensitive;
  }
  
  public int getRefreshTimeout(MoreMotionRequest request) throws ConfigParameterException {
    return refreshTimeout == null ? 0 : refreshTimeout.resolve(request).intValue(0);
  }
  
  public String getResBundleName(MoreMotionRequest request) throws ConfigParameterException {
    return resbundle == null ? null : resbundle.resolve(request).stringValue(null);
  }
  
  public String getUnitName(MoreMotionRequest request) throws ConfigParameterException {
    return unitname == null ? null : unitname.resolve(request).stringValue(null);
  }
  
  public AccessControlDefinition getAccessControlDefinition(MoreMotionRequest request) throws ConfigParameterException {
    return accessControl == null ? null : accessControl.getAccessControlDefinition(request);
  }
  
  public String getPrecond(MoreMotionRequest request) throws ConfigParameterException {
    return precond == null ? null : precond.resolve(request).stringValue(null);
  }
  
  public int getItemsPerPage(MoreMotionRequest request) throws ConfigParameterException {
    return itemsPerPage == null ? 0 : itemsPerPage.resolve(request).intValue(0);
  }
  
  public int getPagesPerBlock(MoreMotionRequest request) throws ConfigParameterException {
    return pagesPerBlock == null ? 0 : pagesPerBlock.resolve(request).intValue(0);
  }
  
  public int getDebugLevel(MoreMotionRequest request) throws ConfigParameterException {
    return debug == null ? 0 : debug.resolve(request).intValue(0);
  }
  
  
  public DataSourceConfig(ConfigUnit unit) throws ConfigurationException {
    super(unit,false);
    parse(unit.getConfigNodeList());
  }
  

  protected String getNodeValue(ConfigNode node) throws DOMException {
    
    String s = node.getValue();
    
    if (s.indexOf("@vof(LANG)") > -1) {
      this.languageSensitive = true;
    }
    return s;
  }
  
  private void parse(ConfigNodeList nodelist) throws ConfigurationException {
    
    for (int i = 0; i < nodelist.size(); i++) {
      
      ConfigNode node = nodelist.get(i);
      
      String name = node.getName();
      
      if (name.equals("scope")) {
        scope = parseScope(node.getValue());
      }
      
      else if (name.equals("accessControl")) {
        accessControl = new AccessControlConfigParameter(node.getElement());
      }
      
      else if (name.equals("unitname")) {
        unitname = new ConfigParameter("unitname", getNodeValue(node));
      }
      
      else if (name.equals("resbundle")) {
        resbundle = new ConfigParameter("resbundle", getNodeValue(node));
      }
      
      else if (name.equals("refreshTimeOut")) {
        refreshTimeout = new ConfigParameter("refreshTimeOut", node.getValue());
      }
      
      else if (name.equals("precond")) {
        precond = new ConfigParameter("precond", node.getValue());
      }
      
      else if (name.equals("itemsPerPage")) {
        itemsPerPage = new ConfigParameter("itemsPerPage", node.getValue());
      }
      
      else if (name.equals("pagesPerBlock")) {
        pagesPerBlock = new ConfigParameter("pagesPerBlock", node.getValue());
      }
      
      else if (name.equals("debug")) {
        debug = new ConfigParameter("debug", node.getValue());
      }
      
      else if (name.equals("class")) {
        className = node.getValue();
      }
      
      else {
        if (params == null) params = new HashMap();
        params.put(name, new ConfigParameter(name, getNodeValue(node))) ;
      }
      
    }
    
    if (className == null) {
      throw new ConfigurationException("DATA_SERVICE_CLASS_NAME_MISSING",new String[]{getName()},null);
    }
    
    
  }
  
  
  private char parseScope(String s) throws ConfigParameterException {
    if (s.equals("Request")) return 'r';
    if (s.equals("Session")) return 's';
    if (s.equals("Application")) return 'a';
    throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{"scope",s},null);
  }
  
  public char getScope() {
    return this.scope;
  }
  
  /**
   * Returns the name of the class responsible to provide data for this Data Source
   * that implements {@link org.moremotion.datasrc.DataService} interface
   */
  public String getClassName() {
    return this.className;
  }
  

}