/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Conventions.java
 *
 * Created on July 14, 2002, 3:38 PM
 */

package org.moremotion.evaluator;

import java.util.Locale;
 
/**
 * This class provides static methods for checking the validity of the given identities or 
 * to convert the given strings to valid identifiers.
 * @author  Mehmet
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Conventions.java 51 2008-03-10 11:46:54Z erkan $
 */
public class Conventions { 
  private static final String 
  fieldChars = "qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM_0123456789",
  invalidEntityChars = "?<>[]{}()$@\"=*&/\\!'^+%~;,|`. ",
  invalidEntityFirstChars =  "-:0123456789";
  
  //Nodes and Items
  private static boolean isValidEntityStartChar(char c) {
    if (invalidEntityChars.indexOf(c) > -1) return false;
    if (invalidEntityFirstChars.indexOf(c) > -1) return false;
    if (c < 33) return false;
    return true;
  }
  
  private static boolean isValidEntityBodyChar(char c){
    if (invalidEntityChars.indexOf(c) > -1) return false;
    if (c < 33) return false;
    return true;
  }
  
  /**
   * Returns <CODE>true</CODE> is the given string is a valid identifier that can be used as an XML tag name.
   */
  public static boolean isValidEntityIdentifier(String id){
    if (id == null || id.length()<1){
      return false;
    }
    
    if (invalidEntityFirstChars.indexOf(id.charAt(0)) != -1){
      return false;
    }

    for (int i=0; i<id.length(); i++){
      char c = id.charAt(i);
      if (invalidEntityChars.indexOf(c) != -1){
        return false;
      }
    }

    return true;
  }
  
  /**
   * Returns <CODE>true</CODE> if the given string is a valid field name. In a valid field name:
   * <UL>
   * <LI>the first character must be one of a-z, A-Z</LI>
   * <LI>and the following characters must be one of a-z, A-Z, 0-9, '_'</LI>
   * </UL>   
   */
  public static boolean isValidFieldName(String name){
    if (name == null || name.length()<1){
      return false;
    }
    
    if (Character.isDigit(name.charAt(0))){
      return false;
    }
    /*
    if (
    name.equals("_id") ||
    name.equals("_uid") ||
    name.equals("_type") ||
    name.equals("_keywords") ||
    name.equals("_path") 
    ) {
      return false;
    }
    */
    for (int i = 0; i < name.length() ; i++){
      if (fieldChars.indexOf(name.charAt(i)) == -1){
        return false;
      }
    }
    
    return true;
  }
  
  /** 
   * Generates a valid identity string, which can be used as a XML tag, from the given string. 
   * The invalid characters in the given string are
   * replaced with &apos;_&apos; character.
   */
  public static String generateValidId(String str){
    if (str == null || str.length() == 0) return null;
    StringBuffer sb = new StringBuffer(str.length());
    char c = str.charAt(0);
    if (isValidEntityStartChar(c)){
      sb.append(c); 
    } else {
      if (isValidEntityBodyChar(c)) {
        sb.append("N" + c); 
      } else {
        sb.append("_");
      }
    }
    
    for (int i=1; i<str.length(); i++){
      c = str.charAt(i);
      if (isValidEntityBodyChar(c)){
        sb.append(c);
      } else {
        sb.append("_");
      }
    }
    String id = sb.toString();
    if (id.length() > 25){
      id = id.substring(0,25);
    }
    
    return id.toLowerCase(Locale.US);
    
  }
  
  private static final String invalidXMLElementNameBodyChars = "?<>[]{}()$@\"=*&/\\!'^+%~;,|`";
  private static final String invalidXMLElementNameFirstChars =  ".-:0123456789?<>[]{}()$@\"=*&/\\!'^+%~;,|`";  
  
  public static String safeXmlNodeName(String str){
    
    if (str == null || str.length() == 0) return null;
    
    char c = str.charAt(0);
    if (c < 33 || invalidXMLElementNameFirstChars.indexOf(c) > -1) {
      return _safeXmlNodeName(str);
    }
    
    for (int i = 1; i < str.length(); i++){
      c = str.charAt(i);
      if (c < 33 || invalidXMLElementNameBodyChars.indexOf(c) > -1) {
        return _safeXmlNodeName(str);
      }
    }
    return str;
    
  }
  
  private static String _safeXmlNodeName(String str){
    
    StringBuffer sb = new StringBuffer(str.length());
    
    char c = str.charAt(0);
    if (c < 33 || invalidXMLElementNameFirstChars.indexOf(c) > -1) {
      sb.append(c); 
    } else {
      sb.append("_");
    }
    
    for (int i = 1; i < str.length(); i++){
      c = str.charAt(i);
      if (c < 33 || invalidXMLElementNameBodyChars.indexOf(c) > -1) {
        sb.append(c);
      } else {
        sb.append("_");
      }
    }
    return sb.toString();
    
  }
  
  
  
  
 /* 
  public static void main(String[] args) throws Exception{
    System.out.println(isValidEntityIdentifier("ama??r_makinas?"));
  }
*/
}
