/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * PageGenerator.java
 *
 * Created on January 9, 2002, 12:50 PM
 */

package org.moremotion.page;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Calendar;
import java.util.Properties;
import java.util.Date;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMException;
import org.moremotion.adom.ADOMManager;
import org.moremotion.adom.ADOMNode;
import org.moremotion.cache.VariablePool;
import org.moremotion.cache.XslTransformer;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.datasrc.ADOMBuilder;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.DataSourceManager;
import org.moremotion.datasrc.DataSrcUtils;
import org.moremotion.datasrc.RefreshDataSource;
import org.moremotion.evaluator.EvalUtils;
import org.moremotion.exception.ExceptionUtils;
import org.moremotion.process.ProcessManager;
import org.moremotion.resource.Resource;
import org.moremotion.resource.ResourceException;
import org.moremotion.resource.ResourceManager;
import org.moremotion.security.AccessControlDefinition;
import org.moremotion.security.SecurityManager;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.Logger;
import org.moremotion.util.MiscUtils;
import org.moremotion.util.XString;
import org.moremotion.servlet.SystemConfig;
import org.w3c.dom.Node;

/**
 * Responsible to generate a page either as XML or HTML depending on the requester browsers XSLT capability.
 *
 * To decide if the browser can perform the XSLT, its user agent information is checked against the provided
 * information with system parameters "xsltCapableBrowsers" and "xsltIncapableBrowsers".
 * <P>
 * This check is performed one time per new session. It is possible to force the Page Generator to reponse as HTML even
 * though the agent is marked as XSLT capable by including "_sendhtml=true" parameter in the request.
 *<P>
 * In case of a pure XML response is required it can be achieved by including "_sendxml=true" parameter in the request.
 * </P>
 * The generate() method of this class called by DisplayPage service which is mapped to "display.doms" URL.
 * A typical call of the display service is <PRE>
 * display.doms?pg=pagename
 * </PRE>
 * The only mandatory parameter <CODE>pg</CODE> specifies the name of the page (without extension) to be displayed.
 * The requested page can be a simple HMTL file or an XSL file.
 * <P>
 * All parameters together:
 * <ul>
 * <li><code>pg</code> - The Name of the page</li>
 * <li><code>loc</code> - The location of the page. It is significant if page name is given relatively without a "/" character
 * at the first position</li>
 * <li><code>pgsep</code> - The page separator character</li>
 * <li><code>_sendhtml</code> - To force HTML response</li>
 * <li><code>_sendxml</code> - To force Pure XML response</li>
 * <li><code>_nocache</code> - To force to include no-cache headers in the response</li>
 * <li><code>_cmd</code> - The name of the command. It must be one of the commands defined in page configuration</li>
 * <li><code>_lang</code> - To change the language of the page. It is significant if the language of the page is set as "ml" (Multi Lingual)
 * in the page configuration.</li>
 * <li><code>_variant</code> - To request a variant of the page.</li>
 * </ul>
 *
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: PageGenerator.java 215 2008-09-13 10:27:24Z erkan $
 */
public class PageGenerator {
  
  // private boolean debug = false;
  
  // private static final String MODULE_NAME = "pagegen";
  
  private static final String XSLT_PI_HEADER_1 = "<?xml-stylesheet type=\"text/xsl\" href=\"";
  private static final String XSLT_PI_HEADER_2 = "\"?>";
  
  private boolean domRequested = false;
  private Node domResult = null;
  
  
  private String getLMDate(CurrentPage currpage) throws PageGenerationException {
    return "?" + currpage.getPageInfo().getPageFile().lastModified();
  }
  
  public void generate(HttpServletRequest request, HttpServletResponse response, CurrentPage currpage, String xml)
  throws PageGenerationException {
    
    String xmlHeader = "";
    
    try {
      String sendhtmlparm = request.getParameter("_sendhtml");
      boolean sendhtml = sendhtmlparm != null && sendhtmlparm.equalsIgnoreCase("true");
      String sendxmlparam = request.getParameter("_sendxml");
      boolean sendxml = sendxmlparam != null && sendxmlparam.equalsIgnoreCase("true");
      
      boolean userAgentXSLTEnabled = isUserAgentXSLTEnabled(request);
      
      String charset = currpage.getEncoding();
      
      xmlHeader = "<?xml version=\"1.0\" encoding=\"" + charset + "\"?>";
      
      if (!sendxml && !sendhtml && !domRequested && userAgentXSLTEnabled) {
        // String xmlSourceWithXSLTPI = addXSLTPI(currpage.getHostRelativeURL(), xml);
        response.setContentType("text/xml; charset=" + charset);
        
        try {
          BufferedWriter bw = new BufferedWriter(
              new OutputStreamWriter(response.getOutputStream(), charset)
              );
          
          bw.write(xmlHeader + getXSLHref(currpage.getHostRelativeURL() + getLMDate(currpage))  + xml);
          bw.flush();
          bw.close();
        } catch (Exception e){}
        
      } else {
        
        if (domRequested) {
          XslTransformer transformer = SystemContext.getXSLTransformerCache().getTransformer(currpage.getPageInfo().getPageFile());
          domResult = transformer.transformToDom(xmlHeader + xml);
          
          return;
        }
        
        if (sendxml) {
          response.setContentType("text/xml; charset=" + charset);
        } else {
          response.setContentType("text/html; charset=" + charset);
        }
        PrintWriter pw = response.getWriter();
        
        XslTransformer transformer = SystemContext.getXSLTransformerCache().getTransformer(currpage.getPageInfo().getPageFile());
        if (sendxml) {
          String s = transformer.transform(xml);
          int pos = s.indexOf("<table ");
          s = s.substring(pos);
          pw.write("<root><![CDATA[" + s + "]]></root>");
        } else {
          pw.write(transformer.transform(xmlHeader + xml));
        }
        pw.flush();
        pw.close();
      }
    } catch(Exception e) {
      e.printStackTrace();
      throw new PageGenerationException(e);
    }
  }
  
  
  public Node generateDom(MoreMotionRequest request, String page) throws PageGenerationException  {
    domRequested = true;
    generate(request, page);
    return this.domResult;
  }
  
  /*
   * Special Request Parameters
   * _variant: (page variant)
   * _nocache:
   * _sendhtml:
   * _sendxml:
   * pg:
   * loc:
   * _pgsep: Page Separator
   * _cmd (or _cdef old):
   * _lang:
   * _originpage: The name of the page that originates the request
   * _refreshds: One or more parameters each to define refresh settings of a data source
   *
   */
  
   /*
    * Built-in Data Sources:
    *
    * request: Contains the request parameters
    * nextpage: (session) contains "Pages-To-Be-Displayed-Next" list
    * pageparts: ??
    * vpool:
    * var:
    * servertime:
    * variants: Variants of the current page
    *
    */
  public void generate(MoreMotionRequest request, String page) throws PageGenerationException {
    
    HttpServletResponse response = request.getResponse();
    
    CurrentPage currpage = new CurrentPage(request, page);
    
    if (currpage.getPageInfo() == null) {
      if (!domRequested) { request.createMessage("moremotion.PageGenerator", "PAGE_NOT_FOUND", new String[]{page},null);
      request.generateDefaultErrorPage(); }
      return;
    }
    
    Logger log = new Logger("PageGenerator");
    
    if (log.LEVEL1) {
      log.print("\n\nIn _generate. page=" + page);
      log.print("Requested File=" + currpage.getPageInfo().getPageFile().getAbsolutePath());
      log.print("Context Relative URL=" + currpage.getContextRelativeURL());
    }
    
    if ( currpage.getPageInfo().isHtml() ) {
      
      /*
       * The requested page is HTML; Redirect it.
       */
      if (log.LEVEL1) log.print("Page extension is .html; redirecting it. [page:" + currpage.getContextRelativeURL() + "]");
      try {
        response.sendRedirect(currpage.getHostRelativeURL());
        return;
      } catch(IOException ioe) {
        throw new PageGenerationException(ioe);
      }
      
    }
    
    if (! currpage.getPageInfo().isXsl() ) {
      request.createMessage("moremotion.PageGenerator", "PAGE_NOT_FOUND", new String[]{page},null);
      request.generateDefaultErrorPage();
      return;
    }
    
    /*
     *
     * The requested page is XSL.
     *
     */
    
    
    PageConfig pageConfig = currpage.getPageInfo().getPageConfig();
    
    if (pageConfig == null) {
      request.createMessage("moremotion.PageGenerator", "PAGE_CONFIGURATION_NOT_AVAILABLE", new String[]{page},null);
      request.generateDefaultErrorPage();
      return;
    }
    
    if (!domRequested && pageConfig.isAjaxSourcePage()) {
      request.createMessage("moremotion.PageGenerator", "PAGE_NOT_FOUND", new String[]{page},null);
      request.generateDefaultErrorPage();
      return;
    }
    
    request.setCurrentPage(currpage);
    
    if (!request.generatingErrorPage()) {
      validateAccess(request, currpage, pageConfig);
    }
    
    RefreshDataSource.refresh(request);
    
    Properties alreadyIncluded = new Properties();
    
    handleCaching(request, pageConfig);
    
   
    PageXMLData xmldata = new PageXMLData();
    if (SystemConfig.generatePageInfo()) {
      xmldata.append(PageUtils.getPageInfoXml(request));
    }
    
    /**
     * 'vpool' Data Request Index in the PageDataRequest[] array 'pdrlist'
     */
    int vpoolPDRIndex = -1;
    
    
    /**
     * 'var' Data Request Index in the PageDataRequest[] array 'pdrlist'
     */
    int varPDRIndex = -1;
    
    currpage.flushNextPages();
    
    // Execute the 'Before-DataSource-Page-Process-Templates' if available
    if (!domRequested && ProcessManager.executePageProcessTemplates(currpage, request, true) == false) {
      /*
       * ProcessManager returned false probably because of a process error.
       * Just return.
       */
      return;
    }
    
    ArrayList cas = new ArrayList();
    
    /*
     * Analyze the datasources (PageDataRequest objects) required by the page
     */
    PageDataRequest[] pdrlist = new PageDataRequest[0];
    
    if (pageConfig.getDataRequests() != null) {
      try {
        pdrlist = pageConfig.getDataRequests().getRequests(request);
      } catch (Exception e) {
        throw new PageGenerationException(e);
      }
    }
    
    for(int i = 0; i < pdrlist.length; i++) {
      
      PageDataRequest pdr = pdrlist[i];
      
      String dsname;
      try {
        dsname = pdr.getDataSourceName(request);
      } catch (ConfigParameterException e) {
        throw new PageGenerationException("ERROR_IN_DSNAME_OF_DATA_REQUEST",new String[]{pdr.getName()},e);
      }
      
      DataSourceConfig dsconfig = null;
      try {
        dsconfig = DataSourceManager.getDataSourceConfig(request, dsname);
      } catch (ConfigurationException e) {
        throw new PageGenerationException(e);
      }
      
      String adomName = dsconfig == null ? dsname : DataSrcUtils.getAdomName(pdr,dsconfig,request.getLanguage());
      if (log.LEVEL1) log.print("ADOM Name: " + adomName);
      
      ADOM adom = null;
      
      if (adomName.equals("request")) {
        adom = makeRequestADOM(request);
        if (log.LEVEL1) log.print("request datasource created");
        
        
      } else if (adomName.equals("vpool")) {
        // Save the PageDataRequest index of 'vpool' datasource
        vpoolPDRIndex = i;
        
        
      } else if (adomName.equals("var")) {
        // Save the PageDataRequest index of 'var' datasource
        varPDRIndex = i;
        
        
      } else if (adomName.equals("servertime")) {
        adom = makeDateADOM(request);
        if (log.LEVEL1) log.print("servertime datasource created");
        
        
      } else if (adomName.equals("variants")) {
        adom = makePageVariantsADOM(request, currpage.getPageInfo());
        if (log.LEVEL1) log.print("'variants' datasource created");
        
      } else {
        
        /*
         * The Data Request does not belong to a built-in data source.
         */
        
        try {

          ADOMBuilder builder = new ADOMBuilder(request,dsconfig,log);
          if (pdr != null) {
            builder.setPageDataRequest(pdr);
            builder.setAdomName(adomName);
          }

          adom = builder.build();

        } catch(ADOMException ae) {
          
          if (domRequested) {
            throw new PageGenerationException(ae);
          }

          if (ae.isRedirectionRequired()) {

            currpage.pushSelfName();
            currpage.flushNextPages();

            generate(request, ae.getRedirectedPage());
            return;
          }

          // ae.printStackTrace();

          String stackTrace = ExceptionUtils.getStackTrace(ae);
          try {
            stackTrace = ResourceManager.replaceResDefinitions(stackTrace,SystemConfig.applicationLanguage());
          } catch (ResourceException re) {
            re.printStackTrace();
          }
          String[] arguments =
              new String[]{page, dsname, DataSrcUtils.getScopeText(dsconfig.getScope()), adomName, dsconfig.getClassName(), stackTrace};

          String details = "" + new Resource("ADOM_GENERATION_ERROR_DETAILS", arguments,"moremotion");
          request.createMessage("moremotion.PageGenerator", "ADOM_GENERATION_ERROR", null, details);
          request.generateDefaultErrorPage();
          return;
        }

      }
      
      /*
       * Ready for formatting
       */
      
      cas.add( new FormatCandidate(adomName, adom, pdr, request.getLanguage())  );
      
      alreadyIncluded.setProperty(dsname, "true");
      
    } 
    
    
    // Execute the 'After-DataSource-Page-Process-Templates' if available
    if (!domRequested && ProcessManager.executePageProcessTemplates(currpage, request, false) == false) {
      /*
       * ProcessManager returned false probably because of a process error.
       * Just return.
       */
      return;
    }
    
    for (int i = 0; i < cas.size(); i++) {
      FormatCandidate fc = (FormatCandidate)cas.get(i);
      ADOM adom = fc.adom;
      if (adom == null) adom = findADOM(request, fc.adomName);
      
      if (adom != null) {
        ADOMFormatter af = new ADOMFormatter(adom, request, fc.pdr);
        xmldata.append( af.format() );
      }
    }
    
    if (vpoolPDRIndex > -1) {
      ADOM adom = makeVPoolADOM(request);
      ADOMFormatter af = new ADOMFormatter(adom, request, pdrlist[vpoolPDRIndex]);
      xmldata.append( af.format() );
    }
    
    if (varPDRIndex > -1) {
      ADOM adom = makeVarADOM(request);
      ADOMFormatter af = new ADOMFormatter(adom, request, pdrlist[varPDRIndex]);
      xmldata.append( af.format() );
    }
    
    // String masterXml = null;
    
    /*
     * Process Forced DataSources.
     */
    
      
    Hashtable forcedDSs = request.getForcedDatasources();
    if (forcedDSs != null) {

      Enumeration en = forcedDSs.keys();
      while (en.hasMoreElements()) {

        String s = (String)en.nextElement();
        String dsname = XString.before(s,'=',"");

        if (alreadyIncluded.getProperty(dsname) == null) {
          String resBundleName = XString.after(s,'=',"");
          ADOM adom = (ADOM)forcedDSs.get(s);

          ADOMFormatter af = new ADOMFormatter(adom, request, dsname, resBundleName);
          xmldata.append( af.format() );
        }
      }
    }

    StringBuffer xbuf = new StringBuffer(1000);
    xbuf.append("<root>");
    xbuf.append( xmldata.getXmlWithoutRootTag() );
    xbuf.append("</root>");
    
    generate(request.getHttpServletRequest(), response, currpage, xbuf.toString());
    
  }
  
  
//  private String addResourceDatasourceXML(String xml, PageDataRequest dataRequest[]) {
//    return null;
//  }
//
  
  private ADOM findADOM(MoreMotionRequest request, String adomName) {
    ADOM adom = request.getRequestADOM(adomName);
    if (adom == null) {
      adom = request.getSessionADOM(adomName);
      if (adom == null) {
        adom = request.getApplicationADOM(adomName);
      }
    }
    return adom;
  }
  
  
  
  private void validateAccess(MoreMotionRequest request, CurrentPage currpage, PageConfig pageConfig) throws PageGenerationException {
    
    request.adjustVarsToPage(currpage.getPageName(true));
    
    if (request.getMethod().equalsIgnoreCase("GET") && currpage.hasCommandDefinitions()) {
      /*
       * Method is 'GET' and the page has command definitions, so check the parameter
       */
      
      String cmd = request.getPlainParameter("_cmd");
      if (cmd == null) {
        cmd = request.getPlainParameter("_cdef");
      }
      
      if (cmd == null) {
        
        CommandDefinition cdef = currpage.getCommandDefinition("*");
        if (cdef == null) {
          throw new PageGenerationException("COMMAND_DEFINITION_PARAMETER_MISSING");
        }
        cdef.checkRequestParameters(request);
        
      } else {
      
        CommandDefinition cdef = currpage.getCommandDefinition(cmd);
        if (cdef == null) {
          throw new PageGenerationException("NO_SUCH_COMMAND_DEFINITION",new String[]{cmd},null);
        }
        cdef.checkRequestParameters(request);
      }
      
    }
    
    
    /*
     * Check access authority if has not checked by the ProcessManager before
     */
    if (request.getAttribute("moremotion.page.authority.already.checked") == null) {
      
      AccessControlDefinition acd = null;
      try {
        acd = pageConfig.getAccessControlDefinition(request);
        
      } catch (ConfigParameterException e) {
        throw new PageGenerationException(e);
      }
      
      if (acd != null) {
        try {
          if (SecurityManager.checkAccessAuthority(request, acd, currpage) == false) {
          /*
           * SecurityManager already redirected to another page. So just return.
           */
            return;
          }
        } catch (Exception ex) {
          throw new PageGenerationException(ex);
        }
      }
      
    }
    
    
  }
  
  private void handleCaching(MoreMotionRequest request, PageConfig pageConfig) throws PageGenerationException {
    
    try {
      
      boolean nocache = false;
      
      if (request.getAllowCaching() == -1) nocache = true;
      else if (request.getAllowCaching() == 1) nocache = false;
      else {
        nocache = pageConfig.getAvoidCaching(request);
        if (!nocache) {
          nocache = request.getPlainParameter("_nocache","false").equalsIgnoreCase("true");
        }
      }
      
      if (nocache) {
        request.setNoCache();
      }
      
    } catch (ConfigParameterException e) {
      throw new PageGenerationException(e);
    }
    
  }

  
  private static ADOMNode createADOMNodeForParameter(String name) {
    if (name.startsWith("pf_")) {
      String s = name.substring(3);
      int x = s.indexOf("_");
      if (x > 0) {
        String seq = s.substring(0,x);
        if (EvalUtils.isNumber(seq)) {
          s = s.substring(x+1);
          if (s.length() > 0) {
            ADOMNode anode = new ADOMNode("pf_" + s);
            anode.addAttribute("seq",seq);
            return anode;
          }
        }
      }
    }
    return new ADOMNode(name);
  }
  
  private ADOM makeRequestADOM(MoreMotionRequest request) {
    
    ADOM adom = request.newRequestADOM("request");
    Enumeration e = request.getParameterNames();
    while(e.hasMoreElements()) {
      String key = (String) e.nextElement();
      // String kvalue = request.getParameter(key);
      ADOMNode node;
      String[] values = request.getParameterValues(key);
      for (int i = 0; i < values.length; i++) {
        String value = values[i];
        if (org.moremotion.evaluator.Conventions.isValidEntityIdentifier(key)) {
          node = createADOMNodeForParameter(key);
        } else {
          node = new ADOMNode("input");
          node.addAttribute("name", key);
        }
        if (value == null) value = "";
        node.setValue(value);
        adom.addNode(node);
      }
      
    }
  
    ADOMNode rootNode = adom;
    rootNode.setNodeValue("ApplicationPath", request.getApplicationPath());
    rootNode.setNodeValue("AuthType", request.getAuthType());
    rootNode.setNodeValue("CharacterEncoding", request.getCharacterEncoding());
    rootNode.setNodeValue("ContentType", request.getContentType());
    rootNode.setNodeValue("ContextPath", request.getContextPath());
    rootNode.setNodeValue("Method", request.getMethod());
    rootNode.setNodeValue("PathInfo", request.getPathInfo());
    rootNode.setNodeValue("PathTranslated", request.getPathTranslated());
    rootNode.setNodeValue("Protocol", request.getProtocol());
    rootNode.setNodeValue("QueryString", request.getQueryString());
    rootNode.setNodeValue("RemoteAddr", request.getRemoteAddr());
    rootNode.setNodeValue("RemoteHost", request.getRemoteHost());
    rootNode.setNodeValue("RemoteUser", request.getRemoteUser());
    rootNode.setNodeValue("RequestURI", request.getRequestURI());
    rootNode.setNodeValue("RequestedSessionId", request.getRequestedSessionId());
    rootNode.setNodeValue("Scheme", request.getScheme());
    rootNode.setNodeValue("ServletPath", request.getServletPath());
    rootNode.setNodeValue("UserAgent", request.getHeader("user-agent"));
    rootNode.setNodeValue("Referer", request.getHeader("referer"));
    
    rootNode.setNodeValue("BrowserIsXSLTCapable", UserAgentAnalyzer.isXSLTCapable(request));
    rootNode.setNodeValue("XSLTInBrowser", "" + UserAgentAnalyzer.isXSLTEnabled(request));
    
    rootNode.setNodeValue("SessionId", request.getSession().getId());
    rootNode.setNodeValue("LanguageCode", request.getLanguage());
    rootNode.setNodeValue("applicationLanguage", SystemConfig.applicationLanguage());
    
    rootNode.setNodeValue("RefreshId", getRefreshId(request));
    
    return adom;
  }
  
  
  private String getRefreshId(MoreMotionRequest request) {
    HttpSession ses = request.getSession();
    String refreshId = (String)ses.getAttribute("moremotion.refresh.id");
    if (refreshId == null) {
      refreshId = ses.getId();
      ses.setAttribute("moremotion.refresh.id",refreshId);
    }
    
    return refreshId;
  }
  
  private ADOM makeVPoolADOM(MoreMotionRequest request) {
    ADOM adom = ADOMManager.newRequestADOM(request, "vpool");
    VariablePool vpool = request.getVPool();
    
    adom.setNodeValues(vpool.getSnapShot());
    return adom;
  }
  
  
  private ADOM makeVarADOM(MoreMotionRequest request) {
    ADOM adom = ADOMManager.newRequestADOM(request, "var");
    adom.setNodeValues(request.getVars().getSessionVars());
    adom.setNodeValues(request.getVars().getPageVars());
    dumpRequestToVarADOM(request,adom);
    adom.setNodeValues(request.getVPool().getSnapShot());
    return adom;
  }
  
  private void dumpRequestToVarADOM(MoreMotionRequest request, ADOM adom) {
    Enumeration e = request.getParameterNames();
    while(e.hasMoreElements()) {
      String key = (String) e.nextElement();
      // String kvalue = request.getParameter(key);
      String[] values = request.getParameterValues(key);
      for (int i = 0; i < values.length; i++) {
        String value = values[i];
        if (org.moremotion.evaluator.Conventions.isValidEntityIdentifier(key)) {
          if (value == null) value = "";
          adom.setNodeValue(key,value);
        }
      }
    }
  }
  
  
  private ADOM makeDateADOM(MoreMotionRequest request) {
    ADOM adom = ADOMManager.newRequestADOM(request, "servertime");
    Calendar cal = java.util.GregorianCalendar.getInstance();
    int zoneOffset = cal.get(Calendar.ZONE_OFFSET);
    Date dt = cal.getTime();
    ADOMNode rootNode = adom;
    rootNode.setNodeValue("time", "" + dt.getTime());
    rootNode.setNodeValue("zoneoffset", "" + zoneOffset);
    rootNode.setNodeValue("timestamp", MiscUtils.timeStamp());
    return adom;
  }
  
  
  private ADOM makePageVariantsADOM(MoreMotionRequest request, PageInfo pageInfo) {
    
    String cl = pageInfo.getPageConfig().getLanguage();
    
    ADOM adom = ADOMManager.newRequestADOM(request, "variants");
    ADOMNode opt = new ADOMNode("option","*");
    opt.addAttribute("value","*");
    adom.addNode(opt);
    
    String[] variants = pageInfo.getVariants();
    if (variants != null) {
      for (int i = 0; i < variants.length; i++) {
        String v = variants[i];
        opt = new ADOMNode("option",v);
        opt.addAttribute("value",v);
        adom.addNode(opt);
      }
    }
    
    return adom;
    
  }
  
  private String getXSLHref(String xslURI) {
    if (xslURI == null) {
      return "";
    }
    return XSLT_PI_HEADER_1 + xslURI + XSLT_PI_HEADER_2;
  }
  
  /**
   * Adds XSL processing instruction to XML source
   */ /*
  private String addXSLTPI(String xslURI, String xmlSource) {
    if (xslURI == null) {
      return xmlSource;
    }
    String xsltPI = XSLT_PI_HEADER_1 + xslURI + XSLT_PI_HEADER_2;
    
    int lastIndex = xmlSource.lastIndexOf("?>");
    if (lastIndex == -1) {
      return xmlSource;
    }
    
    String result = xmlSource.substring(0, lastIndex + 2) +
        xsltPI +
        xmlSource.substring(lastIndex + 2);
    
    return result;
  } */
  
  private boolean isUserAgentXSLTEnabled(HttpServletRequest request) {
    boolean result = false;
    result = UserAgentAnalyzer.isXSLTEnabled(request);
    return result;
  }
  
  private String safeval(String val) {
    if (val == null) return "";
    return val;
  }
  
}

class FormatCandidate {
  protected String adomName;
  protected ADOM adom;
  protected PageDataRequest pdr;
  protected String language;
  protected FormatCandidate(String adomName, ADOM adom, PageDataRequest pdr, String language) {
    this.adomName = adomName;
    this.adom = adom;
    this.pdr = pdr;
    this.language = language;
  }
}