/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * PageManager.java
 *
 * Created on 05 March 2007 Monday, 00:24
 *
 */

package org.moremotion.page;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.moremotion.cache.FileChangeNotification;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.FileUtils;
import org.moremotion.util.XString;
import org.w3c.dom.Document;
import org.moremotion.cache.FileChangeDetector;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: PageManager.java 149 2008-06-11 13:35:46Z erkan $
 */
public class PageManager implements FileChangeNotification {
  
  private static HashMap cache = new HashMap();
  private static PageManager Instance = null;
  
  public PageManager() {
  }
  
  public static synchronized PageManager getInstance() {
    if (Instance == null) Instance = new PageManager();
    return Instance;
  }
  
  /**
   * Creates PageInfo object for the specified page and returns it. Maintains a cache for created objects
   * so next time for the same page and the variant response is quicker.
   * @param request MoreMotionRequest object
   * @param pageName The name of the MoreMotion Page
   */
  public static PageInfo getPageInfo(MoreMotionRequest request, String pageName) throws PageGenerationException {
    return getPageInfo(request, pageName, null);
  }
  
  /**
   * Creates PageInfo object for the specified page and variant and returns it. Maintains a cache for created objects
   * so next time for the same page and the variant response is quicker.
   * @param request MoreMotionRequest object
   * @param pageName The name of the MoreMotion Page
   * @param variant The page variant
   */
  public static PageInfo getPageInfo(MoreMotionRequest request, String pageName, String variant) throws PageGenerationException {
    
    PageInfo result = null;
    
    if (pageName.charAt(0) == '/') {
      pageName = pageName.substring(1);
    }
    
    String key = pageName;
    if (variant != null && variant.length() > 0) {
      key = key + "@" + variant;
    }
    
    if (cache.containsKey(key)) {
      
      result = (PageInfo)cache.get(key);
      
      if (result != null) {
        
        if (result.getPageFile() == null) {
          
          synchronized (result) {
            if (result.getPageFile() == null) {
              File file = FileUtils.getPageFile(request,pageName,variant);
              if (file == null) return null;
              result.setPageFile(file);
            }
          }
          
        }
        
        if (result.getPageConfig() == null) {
          
          synchronized (result) {
            
            File configFile = FileUtils.getPageConfigFile(request,pageName,variant);
            if (configFile != null) {
              
              try {
                Document doc = getDocument(configFile);
                PageConfig pageConfig = new PageConfig(pageName, doc);
                result.setPageConfig(pageConfig);
              } catch (Exception e) {
                throw new PageGenerationException(e);
              }
              
            }
            
          }
        }
        
        return result;
      }
      
    }
    
    
    synchronized ("page.file.key." + key) {
      
      if (!cache.containsKey(key)) {
        
        File file = FileUtils.getPageFile(request,pageName,variant);
        if (file != null) {
          
          // Register the page for change detection
          FileChangeDetector.getInstance().addFile(file,PageManager.getInstance());
          
          result = new PageInfo();
          result.setPageFile(file);
          
          File configFile = FileUtils.getPageConfigFile(request,pageName,variant);
          if (configFile != null) {
            
            FileChangeDetector.getInstance().addFile(configFile,PageManager.getInstance());
            
            try {
              Document doc = getDocument(configFile);
              PageConfig pageConfig = new PageConfig(pageName, doc);
              result.setPageConfig(pageConfig);
              result.setPageConfigFile(configFile);
            } catch (Exception e) {
              throw new PageGenerationException(e);
            }
            
          }
          
          cache.put(pageName,result);
        }
      }
    }
    
    return result;
  }
  
  private static Document getDocument(File file) throws Exception {
    DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
    return builder.parse(file);
  }
  
  
  protected static String[] getVariants(File pageFile) {
    
    try {
      
      String pageFileName = pageFile.getAbsolutePath();
      File dir = null;
      String special = File.separator + "_";
      if (pageFile.getName().startsWith("_.")) {
        dir = pageFile.getParentFile();
      } else {
        int x = pageFileName.lastIndexOf('.');
        if (x > 0) {
          dir = new File(pageFileName.substring(0,x));
        } else dir = pageFile; // actually should not happen
      }
      
      String[] variants = null;
      if (variants == null) {
        if (dir.exists()) {
          ArrayList al = new ArrayList();
          File[] files = dir.listFiles();
          for (int i = 0; i < files.length; i++) {
            File f = files[i];
            String fn = f.getAbsolutePath();
            if (f.getName().startsWith("_.")) {
              String vn = f.getName().substring(2);
              vn = XString.before(vn,'.');
              al.add(vn);
            }
          }
          variants = new String[al.size()];
          al.toArray(variants);
        }
      }
      return variants;
      
    } catch (Exception e) {}
    
    return null;
  }
  
 
  public void doChange(File f) {
    
    boolean isPageFile = f.getName().endsWith(".xsl") || f.getName().endsWith(".html");
    String fPath = f.getAbsolutePath();
    if (isPageFile) {
      int lastDot = fPath.lastIndexOf('.');
      if (lastDot > -1) {
        fPath = fPath.substring(0,lastDot);
      }
    }
    
    Iterator itr = cache.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      PageInfo pageInfo = (PageInfo)entry.getValue();
      
      File pageFile = pageInfo.getPageFile();
      if (isPageFile && pageFile != null && pageFile.getAbsolutePath().startsWith(fPath) ) {
        pageInfo.setPageFile(null);
        
      } else {
        File pageConfigFile = pageInfo.getPageConfigFile();
        if (pageConfigFile != null && pageConfigFile.getAbsolutePath().equals(fPath)) {
          pageInfo.setPageConfig(null);
        }
        
      }
    }
    
  }
  
  public void doDelete(File f) {
    doChange(f); // The necessary action to take is the same with doChange().
  }
  
}
