/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * UserAgentAnalyzer.java
 *
 * Created on January 7, 2002, 12:32 PM
 */

package org.moremotion.page;

import java.util.ArrayList;
import java.util.StringTokenizer;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import org.moremotion.servlet.SystemConfig;

/** Analyzes user agent and decides if the requester browser is capable of performing XSL Transformation (XSLT) properly.
 * <P>The XSL capable browsers are defined in two system parameters by means of
 * specifying "|" delimited substrings that are supposed to or must not exist in the User-Agent properties of the browsers. 
 * <P>These system parameters are:<BR>
 * <UL>
 * <LI><CODE>xsltCapableBrowsers</CODE>. If one of the "|" delimited substrings exists in the User-Agent<BR> then
 * the browser is considered to be XSLTCapable. </LI>
 * <LI><CODE>xsltInCapableBrowsers</CODE>. If one of the "|" delimited substrings
 * exist in the User-Agent then the browser is NOT XSLTCapable.</LI>
 * <PRE>
 * Examples:
 * &lt;param name="xsltCapableBrowsers"&gt;MSIE 6.|MSIE 7.&lt;/param&gt;
 * &lt;param name="xsltInCapableBrowsers"&gt;Opera&lt;/param&gt;
 *</PRE>
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: UserAgentAnalyzer.java 51 2008-03-10 11:46:54Z erkan $
 */
public class UserAgentAnalyzer {
  
  private static String[] validBrowsers = new String[0];
  private static String[] invalidBrowsers = new String[0];

  private static String getCookieValue(HttpServletRequest request, String cookieName) {
    Cookie[] cookies = request.getCookies();
    for (int i=0; cookies != null && i < cookies.length; i++) {
      if (cookies[i].getName().equals(cookieName)) {
        return cookies[i].getValue();
      }
    }
    return null;
  }
  
  
  public static boolean isXSLTEnabled(HttpServletRequest request) {
    HttpSession session = request.getSession();
    String canxslt = (String)session.getAttribute("XSLTEnabled");
    if (canxslt != null) {
      if (canxslt.equals("true")) {
        return true;
      } else {
        return false;
      }
    }
    
    String cookie = getCookieValue(request, "XSLTEnabled");
    if (cookie != null && cookie.equalsIgnoreCase("false")) {
      session.setAttribute("XSLTEnabled","false");
      return false;
    }
    
    canxslt = isXSLTCapable(request); 

    session.setAttribute("XSLTEnabled",canxslt);
    return canxslt.equals("true");

  }
  
  private static String[] parseUserAgentString(String value) {
    ArrayList a = new ArrayList();
    StringTokenizer tok = new StringTokenizer(value, "|");
    while (tok.hasMoreTokens()) {
      String s = tok.nextToken();
      a.add(s);
    }
    
    String[] result = new String[a.size()];
    a.toArray(result);
    return result;
  }
  
  
  public static void init() {
  
    validBrowsers = parseUserAgentString(SystemConfig.getParameter("xsltCapableBrowsers",""));
    invalidBrowsers = parseUserAgentString(SystemConfig.getParameter("xsltIncapableBrowsers",""));
    
  };
  
  
  public static String isXSLTCapable(HttpServletRequest request) {
    HttpSession session = request.getSession();
    String canxslt = (String)session.getAttribute("XSLTCapable");
    if (canxslt != null) {
      if (canxslt.equals("true")) {
        return "true";
      } else {
        return "false";
      }
    }
    
    String userAgentValue = request.getHeader("user-agent");

    for (int i = 0; i < invalidBrowsers.length; i++) {
      String s = invalidBrowsers[i];
      if (userAgentValue.indexOf(s) > -1) {
        session.setAttribute("XSLTCapable","false");
        return "false";
      }
    }
    
    for (int i = 0; i < validBrowsers.length; i++) {
      String s = validBrowsers[i];
      if (userAgentValue.indexOf(s) > -1) {
        session.setAttribute("XSLTCapable","true");
        return "true";
      }
    }

    session.setAttribute("XSLTCapable","false");
    return "false";

  }  
  
}