/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ProcessRecord.java
 *
 * Created on October 16, 2003, 7:48 PM
 */

package org.moremotion.process;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Represents a Process Record that is contained in a {@link org.moremotion.process.FormProcessBlock FormProcessBlock}.
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ProcessRecord.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ProcessRecord {

  private ProcessBlock owner;
  private int index = 0;
  
  protected ProcessRecord(ProcessBlock owner) {
    this.owner = owner;
  }
  
  protected void setIndex(int index) {
    this.index = index;
  }
  
  public int getIndex() {
    return this.index;
  }
  
  public ProcessBlock getBlock() {
    return this.owner;
  }
  
  
  /**
   * Returns the value of the specified field of this process record. 
   * @param name The name of the name. It can be custom name or a block name.
   * @param defValue The value to be returned if specified name is not found.
   * @return The value of the custom name or block name or the default value.
   */
  public String getField(String name, String defValue) {
    String s = owner.getField(name,index);
    return s == null ? defValue : s;
  }
  
  /**
   * Returns the value of the specified field of this process record as integer value. 
   * @param name The name of the name. It can be custom name or a block name.
   * @param defValue The value to be returned if specified name is not found or not a integer value.
   * @return The value of the custom name or block name or the default value.
   */
  public int getFieldAsInt(String name, int defValue) {
    String s = owner.getField(name,index);
    
    if (s == null) return defValue;
    try {
      return Integer.parseInt(s);
    } catch(Exception e) {
      return defValue;
    }
  }
  
  /**
   * Returns the value of the specified field of this process record as double value. 
   * @param name The name of the name. It can be custom name or a block name.
   * @param defValue The value to be returned if specified name is not found or not a valid double.
   * @return The value of the custom name or block name or the default value.
   */
  public double getFieldAsDouble(String name, double defValue) {
    String s = owner.getField(name,index);
    
    if (s == null) return defValue;
    try {
      return Double.parseDouble(s);
    } catch(Exception e) {
      return defValue;
    }
  }
  
  /**
   * Returns the value of the specified field of this process record. 
   * @param name The name of the name. It can be custom name or a block name.
   * @param defValue The value to be returned if specified name is not found.
   * @return The value of the custom name or block name or the default value.
   */
  public boolean getFieldAsBoolean(String name, boolean defValue) {
    
    String s = owner.getField(name,index);
    
    if (s == null) return defValue;
    
    if (s.equalsIgnoreCase("true") || s.equals("1")) {
      return true;
    }
    
    if (s.equalsIgnoreCase("false") || s.equals("0")) {
      return false;
    }
    
    return defValue;
  }
  
  /**
   * Creates a custom name in this record and sets its value.
   * @param name The name of the custom name to be set
   * @param value Value to set
   */
  public void setField(String name, String value) {
    owner.setField(name, value, index);
  }
  
  /**
   * Sets an object with the specified name in this record.
   * @param name The name of the object to be set
   * @param object The object to set
   */ 
  public void setObject(String name, Object object) {
    owner.setObject(name, object, index);
  }
  
  /**
   * Returns the object with the specified name existing in this record.
   */
  public Object getObject(String name) {
    return owner.getObject(name,index);
  }
  
  /**
   * Returns <CODE>true</CODE> if this record is marked as modified.
   */
  public boolean isModified() {
    return owner.isModified(index);
  }
  
  /**
   * Returns <CODE>true</CODE> if the given field in the record is modified.
   */
  public boolean isFieldModified(String name) {
    return owner.isFieldModified(name, index);
  }
  
  /**
   * Returns <CODE>true</CODE> if this records is marked as selected.
   */
  public boolean isSelected() {
    return owner.isSelected(index);
  }
  
  public String toString() {
    return "index : " + index + "; block { " + owner.toString() + "}";
  }
  
}
