/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * JdbcConnection.java
 *
 * Created on 24 July 2004 Saturday, 01:04
 */
package org.moremotion.util;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.config.ConfigurationException;

import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemContext;

/**
 * This class provides static methods for obtaining Data Base Connections.
 * <P>If JNDI Datasource parameter is specified, either as parameter or in the Data Source Configuration,
 * the methods first try to get the connection through JNDI, if they fail then they load the Jdbc Driver Class
 * and gets the connection from the Driver Manager with the parameters defined in the RelDB Datasource.
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: JdbcConnection.java 215 2008-09-13 10:27:24Z erkan $
 */
public class JdbcConnection {
  
  private static final String REQUEST_CONNECTION_POOL = "moremotion.jdbc.connection.pool";

  /**
   * Returns a JdbcConnection connection.
   * 
   * @param conName The name of the jdbcConnection as defined in configuration.
   * @return The obtained connection.
   * @throws Exception if the cannot be obtained.
   */
  public static java.sql.Connection getConnection(String conName) throws JdbcConnectionException {
    return getConnection(null, conName);
  }

  /**
   * Returns a Jdbc Connection.
   * 
   * @param request The MoreMotionRequest object
   * @param conName The name of the jdbcConnection as defined in configuration.
   * @return The obtained connection.
   * @throws Exception if no connection can be obtained.
   */
  public static java.sql.Connection getConnection(MoreMotionRequest request, String conName) throws JdbcConnectionException {

    Config config;
    try {
      config = ConfigManager.getConfig("jdbcConnection", conName);
    } catch (ConfigurationException ex) {
      throw new JdbcConnectionException("DATASOURCE_HAS_INVALID_PARAMS", new String[]{conName}, ex);
    }

    if (config == null) {
      throw new JdbcConnectionException("JDBC_CON_CONFIG_HAS_INVALID_PARAMS", new String[]{conName}, null);
    }
    return getConnection(request, config);
  }

  private static String safe(String s) {
    if (s == null) {
      return "";
    }
    return s;
  }

  /**
   * Returns a Jdbc connection. If a connection was obtained before with the
   * same parameters during the same request it will be reused.
   * 
   * @param request The MoreMotionRequest object
   * @param config Config object
   * @return The obtained connection.
   * @throws Exception if the cannot be obtained.
   */
  public static java.sql.Connection getConnection(MoreMotionRequest request, Config config) throws JdbcConnectionException {

    try {

      String jndiName = null;
      String jdbcDriver = null;
      String databaseURL = null;
      String userName = null;
      String password = null;
      boolean autoCommit = true;
      String firstSQL = null;
      String lastSQL = null;

      
      if (request != null) {
        jndiName = config.getParameter("jndids").resolve(request).stringValue(null);
        jdbcDriver = config.getParameter("driver").resolve(request).stringValue(null);
        databaseURL = config.getParameter("dburl").resolve(request).stringValue(null);
        userName = config.getParameter("user").resolve(request).stringValue(null);
        password = config.getParameter("password").resolve(request).passwordValue("");
        autoCommit = config.getParameter("autoCommit").resolve(request).booleanValue(true);
        firstSQL = config.getParameter("firstSQL").resolve(request).stringValue(null);
      } else {
        jndiName = config.getParameter("jndids").stringValue(null);
        jdbcDriver = config.getParameter("driver").stringValue(null);
        databaseURL = config.getParameter("dburl").stringValue(null);
        userName = config.getParameter("user").stringValue(null);
        password = config.getParameter("password").passwordValue("");
        autoCommit = config.getParameter("autoCommit").booleanValue(true);
        firstSQL = config.getParameter("firstSQL").stringValue(null);
      }

      lastSQL = config.getParameter("lastSQL").stringValue(null);
      String conKey = safe(jndiName) + safe(jdbcDriver) + safe(databaseURL) + safe(userName) + safe(password);

      RequestConnectionPool rcpool = null;

      if (request != null) {

        rcpool = getRequestConnectionPool(request);
        Connection conn = (Connection) rcpool.getConnection(conKey);
        if (conn != null && conn instanceof Connection) {
          if (!conn.isClosed()) {
            return conn;
          } else {
//          removeConnectionFromPool(conn);
          }
        }
      }

      java.sql.Connection result = null;

      if (jndiName != null) {

        /*
         * A JNDI DataSource is defined. We will try to find the datasource within the naming.context.
         * org.moremotion.servlet.SystemConfig tries to create initial contextes for
         * InitialContext, InitialDirContext and InitialLdapContext
         */
        try {
          javax.naming.Context context = SystemContext.getInitialContext();

          javax.sql.DataSource jndids = (javax.sql.DataSource) context.lookup(jndiName);
          if (userName == null) {
            result = jndids.getConnection();
          } else {
            result = jndids.getConnection(userName, password);
          }
          if (result != null) {
            result.setAutoCommit(autoCommit);
            if (request != null) {
              rcpool.add(conKey, result, lastSQL);
              if (firstSQL != null && firstSQL.trim().length() > 0) {
                executeSQL(request, result, firstSQL);
              }
            }
            return result;
          }
        } catch (Exception e) {
          e.printStackTrace();
          result = null;
        }

      }

      // Use Driver Manager to get a connection
      Class.forName(jdbcDriver);
      if (userName == null || password == null) {
        result = DriverManager.getConnection(databaseURL);
      } else {
        result = DriverManager.getConnection(databaseURL, userName, password);
      }
      if (result != null) {
        result.setAutoCommit(autoCommit);
        if (request != null) {
          rcpool.add(conKey, result, lastSQL);
          if (firstSQL != null && firstSQL.trim().length() > 0) {
            executeSQL(request, result, firstSQL);
          }
        }

      }

      return result;

    } catch (Exception ex) {
      throw new JdbcConnectionException(ex);
    }
  }

  private static void executeSQL(MoreMotionRequest request, Connection conn, String sql) throws Exception {

    MScriptResolver resolver = new MScriptResolver(sql, request.getSymbolResolver(), ';');
    resolver.setSqlInjectionCheck(true);

    Statement stmt = null;
    CallableStatement cstmt = null;

    try {

      String querypart = resolver.nextStatement();

      while (querypart != null) {

        if (querypart.startsWith("selecT")) {
          /*
           * Special Query
           */
          querypart = JdbcUtils.arrangeSpecialSQL(querypart);
          querypart = querypart.replaceAll("vpool:", "vpool_");

          if (stmt == null) {
            stmt = conn.createStatement();
          }

          ResultSet specialRs = stmt.executeQuery(querypart);

          if (specialRs != null) {
            JdbcUtils.setFields(specialRs, null, request, null);
          }

        } else if (querypart.startsWith("{") && querypart.endsWith("}")) {

          /*
           * A Stored Procedure Call
           */
          if (cstmt != null) {
            cstmt.close();
          }
          cstmt = conn.prepareCall(querypart);
          cstmt.execute();

        } else {

          if (stmt == null) {
            stmt = conn.createStatement();
          }
          stmt.execute(querypart);

        }

        querypart = resolver.nextStatement();
      }

      resolver.setSqlInjectionCheck(false);


    } finally {

      try {
        if (stmt != null) {
          stmt.close();
        }
        if (cstmt != null) {
          cstmt.close();
        }
      } catch (SQLException sqle) {
        sqle.printStackTrace();
      }

    }

  }

  public static void releaseConnection(java.sql.Connection conn) {

    // Not a managed connection. Close it.
    try {
      if (!conn.isClosed()) {
        conn.close();
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private static RequestConnectionPool getRequestConnectionPool(MoreMotionRequest request) {

    RequestConnectionPool rcpool = (RequestConnectionPool) request.getAttribute(REQUEST_CONNECTION_POOL);
    if (rcpool == null) {
      rcpool = new RequestConnectionPool();
      request.setAttribute(REQUEST_CONNECTION_POOL, rcpool);
    }
    return rcpool;
  }

  public static void releaseRequestConnections(MoreMotionRequest request) throws Exception {

    RequestConnectionPool rcpool = (RequestConnectionPool) request.getAttribute(REQUEST_CONNECTION_POOL);
    if (rcpool == null) {
      return;
    }
    rcpool.top();
    while (rcpool.next()) {
      Connection conn = rcpool.getConnection();
      String sql = rcpool.getSQL();
      try {
        if (sql != null) {
          executeSQL(request, conn, request.resolve(sql));
        }
      } finally {
        if (conn instanceof java.sql.Connection) {
          releaseConnection(conn);
        }
      }
    }

  }
}

class RequestConnectionPool {

  private HashMap pool = new HashMap();
  private Iterator itr = null;
  private Map.Entry e = null;

  protected void add(String key, Connection conn, String sql) {
    if (pool.get(key) != null) {
      return;
    }
    pool.put(key, new CPEntry(conn, sql));
  }

  protected Connection getConnection(String key) {
    CPEntry cpe = (CPEntry) pool.get(key);
    return cpe == null ? null : cpe.conn;
  }

  protected void top() {
    itr = pool.entrySet().iterator();
  }

  protected boolean next() {
    if (itr.hasNext()) {
      e = (Map.Entry) itr.next();
      return true;
    } else {
      return false;
    }
  }

  protected Connection getConnection() {
    return ((CPEntry) e.getValue()).conn;
  }

  protected String getSQL() {
    return ((CPEntry) e.getValue()).sql;
  }
}

class CPEntry {

  Connection conn;
  String sql;

  CPEntry(Connection conn, String sql) {
    this.conn = conn;
    this.sql = sql;
  }
}

