/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ItemParser.java
 *
 * Created on November 14, 2003, 5:38 PM
 */

package org.moremotion.util;

/**
 * String Parser parses the items in a string bwith the specified delimiter. The difference of this class from the
 * java.util.StringTokenizer is that this class ignores the delimiters enclosed with (), [], {} , '', "" characters.
 * <P><B>Example:</B><PRE>
 * StringParser sp = new StringParser("(xxx,yyy)',',[(qqq,www),(ppp,rrr)],zzz",',');
 * sp.nextItem(); // returns "(xxx,yyy)','"
 * sp.nextItem(); // returns "[(qqq,www),(ppp,rrr)]
 * sp.nextItem(); // returns "zzz"
 * </PRE>
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: StringParserEx.java 51 2008-03-10 11:46:54Z erkan $
 */
public class StringParserEx {
  
  
  private int csr = 0;
  private String str = "";
  private String[] dlms;
  private char[] oencls;
  private char[] cencls;
  private char[] bracebuffer;
  private int icount = -1;
  private boolean trim = false;
  private int dlmIndx = -1;
  
  
  /**
   * Creates a new StringParser
   */
  public StringParserEx(String str, String[] dlms, char[] oencls, char[] cencls) {
    this.str = str;
    this.dlms = dlms;
    this.oencls = oencls;
    this.cencls = cencls;
    bracebuffer = new char[20];
    csr = 0;
  }
  
  /**
   * Returns <CODE>true</CODE> if end of string is reached.
   */
  public boolean hasMore() {
    return csr < str.length();
  }
  
  
  public void trimmed() {
    this.trim = true;
  }
  
  
  /**
   * Returns the first item in the string.
   */
  public String firstItem() {
    csr = 0;
    return nextItem();
  }
  
  private int match(int pos) {
    
    boolean f = true;
    
    if (pos > 0) {
      f = false;
      char c = str.charAt(pos-1);
      for (int i = 0; i < oencls.length; i++) {
        if (oencls[i] == c) {
          f = true;
          break;
        }
      }
      if (!f) return -1;
    }
    
    int di = 0;
    for (; di < dlms.length; di++) {
      String dlm = dlms[di];
      int p = pos;
      f = true;
      for (int j = 0; j < dlm.length(); j++) {
        if (p == str.length()) {
          f = false;
          break;
        }
        char c = str.charAt(p);
        if (c != dlm.charAt(j)) {
          f = false;
          break;
        }
        p++;
      }
      
      
      if (f) {
        
        if (p >= str.length()) return str.length();
        char c = str.charAt(p);
        for (int k = 0; k < cencls.length; k++) {
          if (cencls[k] == c) {
            dlmIndx = di;
            return p+1;
          }
        }
      }
      
    }
    return -1;
  }
  
  /**
   * Returns the next item in the string.
   */
  public String nextItem() {
    
    dlmIndx = -1;
    
    if (!hasMore()) return "";
    
    int braceindex = -1;
    char quote = ' ';
    int i = csr;
    int j = i;
    for (; i < str.length(); i++) {
      char cc = str.charAt(i);
      char ch = ' ';
      if (quote == ' ' && braceindex == -1) {
        int m = match(i);
        if (m > -1) {
          j = m;
          break;
        }
      }
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      }
      else if (quote == cc) quote = ' ';
      if (quote == ' ') {
        if (cc == '(') ch = ')';
        else if (cc == '[') ch = ']';
        else if (cc == '{') ch = '}';
        else if (braceindex > -1 && cc == bracebuffer[braceindex]) braceindex--;
      }
      if (ch != ' ') {
        bracebuffer[++braceindex] = ch;
        ch = ' ';
      }
      
    }
    
    String result = str.substring(csr,i);
    csr = i > j ? i : j;
    
    if (trim) return result.trim();
    return result;
  }
  
  /**
   * Moves the cursor to the first position.
   */
  public void reset() {
    csr = 0;
  }
  
  public String getDelimiter() {
    return dlmIndx == -1 ? "" : dlms[dlmIndx];
  }
  
  /**
   * Returns the next item as <CODE>int</CODE> and if next item is not a number it returns the default value.
   */
  public int nextItemAsInt(int defValue) {
    try {
      return Integer.parseInt(this.nextItem());
    } catch(Exception e){
      return defValue;
    }
  }
  
  /**
   * Returns the next item as <CODE>boolean</CODE>. If the next item is not "true" returns <CODE>false</CODE>.
   */
  public boolean nextItemAsBoolean() {
    return this.nextItem().equalsIgnoreCase("true") ? true : false;
  }
  
  public String nextEnclosed() {
    String str = this.nextItem();
    if (str.length() > 2) {
      str = str.substring(1,str.length()-1);
    }
    return str;
  }
  
  
  public static void main(String[] args) {
    
    
    String s = "A or   (B  or  B )  AND C";
    StringParserEx p = new StringParserEx(s,new String[]{"or","OR","and","AND"},new char[]{' ','('}, new char[]{' ',')'});
    p.trimmed();
    while (p.hasMore()) {
      System.out.println(p.nextItem());
    }
  }  
  
  
}
